/*
 * Copyright (C) 2005, 2007, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef BASETOOLS_H
# define BASETOOLS_H

# include <string>
# include <vector>
# include <set>

# include "workerdata.h"

/**
 * 
 */
void* createThumbs(void* wd);

/**
 * 
 */
void* copyFiles(void* wd);

class WorkThumbCreate
{
public:
  std::vector<std::string> filelist;
  int                      nMaxW;
  int                      nMaxH;
  float                    flQual; // 0 .. 1.0
  int                      nType;
};

class BaseTools
{
public:
  
  /**
   * 
   */
  static void getShrinkSize(int w, int h, int& wmax, int& hmax);
  
  /**
   * quality: 0 .. 1.0
   */
  static int createThumb(std::string dest, std::string src, 
                         float quality, int type,
                         int wmax, int hmax);
  
  /**
   * returns 0 if everything is ok
   */
  static int createThumbs(WorkerData* wd);
  
  /**
   * returns 0 if everything is ok
   */
  static int copyFiles(WorkerData* wd);
  
  /**
   * Translates reserved characters in a keyword to underscores.
   */
  static std::string Keyword(std::string in);
  
  /**
   * Trim string 'in' which contains a keywords list (user input!) and return the result.
   * Conversions:
   *   newline        -> ','
   *   any whitespace -> ' '
   *   ",,"           -> ','
   *   "  "           -> ' '
   * 
   * The result will be a string with keywords separated by ','. Leading and trailing 
   * whitespace is stripped from keywords. Examples:
   *   keyword1
   *   keyword1,keyword2
   *   keyword1,keyword2,keyword3
   */
  static std::string KeywordList_String(std::string in);
  
  /**
   * 'in' contains a keywords list (user input), is converted to a set and returned.
   */
  static std::set<std::string> KeywordList_Set(std::string in);
  
  static std::string getFilePath(std::string relname);
};


#endif
