/*
 * Copyright (C) 2006, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "batch.h"

#include "treeitem.h"
#include "treepicbase.h"
#include "treeview.h"

void BatchProc::kw_add(std::vector<TreeItem*> list, std::set<std::string> kwset)
{
  unsigned int size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
    list[cnt]->kw_add(kwset, false);
}

void BatchProc::editGimp(std::vector<TreeItem*> list, std::string basepath)
{
  unsigned int size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    if (list[cnt]->getType() == TreeItem::PIC)
      ((TreePicBase*)list[cnt])->editGimp(basepath);
  }
}

void BatchProc::getPicPath(std::vector<TreeItem*> list,
                           TreeView* data,
                           std::vector<std::string>& srclist, int nSize)
{
  unsigned int size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    if (list[cnt]->getType() == TreeItem::PIC)
      srclist.push_back(data->getPicPath(list[cnt], nSize));
  }
}

void BatchProc::timeshift(std::vector<TreeItem*> list,
                          int nSec)
{
  unsigned int size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    if (list[cnt]->getType() == TreeItem::PIC)
      ((TreePicBase*)list[cnt])->timeshift(nSec);
  }
}

int BatchProc::deleteItems(std::vector<TreeItem*> list,
                           TreeView* data,
                           bool fDelFiles)
{
  int nRetCode = 0;
  unsigned int size = list.size();
  
  for (unsigned int cnt=0; cnt<size; cnt++)
  {
    nRetCode |= data->deleteItem(list[cnt], fDelFiles);    
  }  
  
  return(nRetCode);
}
