/*
 * Copyright (C) 2006, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef BATCH_H
#define BATCH_H

#include <string>
#include <vector>
#include <set>

class TreeItem;
class TreeView;

/**
 * Some actions have been taken out of TreeView to make it somewhat smaller. 
 * Copying marked elements to a list and then browsing this list is bad in terms
 * of memory usage and speed, but that does not matter for the actions in here.
 * They are used infrequently and are very fast anyway.
 * 
 * 
 * All in all, doing this just creates more code and takes little code from TreeView. 
 * Maybe it will pay out when adding more features.
 */
class BatchProc
{
  public:
   
   static void kw_add(std::vector<TreeItem*> list, 
                      std::set<std::string>  kwset);
   
   static void editGimp(std::vector<TreeItem*> list, 
                        std::string basepath);
   
   static void getPicPath(std::vector<TreeItem*> list, 
                          TreeView* data,
                          std::vector<std::string>& srclist, int nSize);
   
   static void timeshift(std::vector<TreeItem*> list,
                         int nSec);
   
   /**
    * Returncodes: see @ref TreeView::deleteItem, or'ed together
    */
   static int deleteItems(std::vector<TreeItem*> list,
                          TreeView* data,
                          bool fDelFiles);
};
#endif
