/*
 * Copyright (C) 2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef BLOCKINGTASK_H
#define BLOCKINGTASK_H

#include "../config.h"

#include <string>
#include <list>

class BlockingTask
{
public:
  
  /**
   * returns zero on success
   */
  virtual int Exec() = 0;
};

class BlockingTask_System : public BlockingTask
{
public:  
  BlockingTask_System(std::string strCmd);
  virtual int Exec();
  
private:
  std::string strCmd;
};

class BlockingTask_Rotate : public BlockingTask
{
public:
  BlockingTask_Rotate(std::string strPath, int nDegrees);
  virtual int Exec();
  
private:
  std::string strPath;
  int nDegrees;
};

class BlockTaskThread
{
public:
  BlockTaskThread();
  int nDone;
  int nSize;
  int nErrCnt;
  bool fPause;
  bool fPauseUser;
  bool fCancel;
  std::list<BlockingTask*> tasks;
};

class BlockTaskManager
{
public:
  void Add(BlockingTask* pTask);
  bool Status(int& nDone, int& nSize);
  
  /**
   * -1: everything done
   *  0: unchanged
   *  1: changed from idle to active
   */
  int Step();
  
  void Pause(bool fPause);
  void PauseUser(bool fPause);
  void Cancel() { threadData.fCancel = true; };
private:
  std::list<BlockingTask*> tasks;
  BlockTaskThread threadData;
};

#endif
