// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "links.h"

#include <iostream>
#include "treeview.h"

void LinkInfo::newLinkableItem(TreeItem* item)                /*{{{*/
{
  unsigned int uID  = item->getID();
  
  unsigned int uIdx = getIndexNear(uID);
  
  if (uIdx < lil.size() && lil[uIdx].getID() == uID)
  {
    // error: this item is part of the list already
  }
  else
  {
    LinkableItemInfo tmp;
    tmp.setID(uID);
    tmp.setLinkCnt(0);
    tmp.ptr = item;

    if (lil.size() == 0)          
      lil.push_back(tmp);
    else
    {
      if (uIdx >= lil.size())
        lil.push_back(tmp);
      else
        lil.insert(lil.begin()+uIdx, tmp);
    }    
  }  
}
/*}}}*/

unsigned int LinkInfo::getIndexNear(unsigned int uID)         /*{{{*/
{
  unsigned int size  = lil.size();
  unsigned int index = size  / 2;
  unsigned int step  = index / 2;

  if (size == 0)
    return(0);

  while (step > 0)
  {
    if (lil[index].getID() > uID)
    {
      index -= step;
      if (index < 0)
        index = 0;
    }
    else
    {
      index += step;
      if (index >= size)
        index = size-1;
    }
    step >>= 1;
  }
  
  while (lil[index].getID() > uID && index > 0)
    index--;
  
  while (lil[index].getID() < uID && index < size-1)
    index++;  
  
  if (lil[index].getID() < uID && index == size-1)
    index = size;
  
  return(index);
}
/*}}}*/

void LinkInfo::showList()                                     /*{{{*/
{
  std::cout << "--- links ------------------------------------\n";
  for (unsigned int n=0; n<lil.size(); n++)
  {
    std::cout << "  " << lil[n].getID() << " -> " << lil[n].ptr << " (" << lil[n].getLinkCnt() << ") " << lil[n].ptr->getText() << "\n";
  }
  std::cout << "----------------------------------------------\n";  
}
/*}}}*/

int LinkInfo::countLinkCount(TreeItem**            ptr,
                             unsigned int          uIDToCount,
                             TreeItem::eItemType   expectedType,
                             bool                  fIncr)     /*{{{*/
{
  unsigned int    pos;
  int             nRetCode = 0;
  
  if (uIDToCount == 0)
  {
    *ptr = (TreeItem*)0;
    return(0);
  }
  
  pos = getIndexNear(uIDToCount);
    
  if (pos < lil.size() && lil[pos].getID() == uIDToCount)
  {    
    if (expectedType != TreeItem::ANY && lil[pos].ptr->getType() != expectedType)
    {
      parent->itemWrongType(uIDToCount, expectedType);
      nRetCode |= eLiErrLink;
      *ptr = (TreeItem*)0;
    }    
    else
    {
      *ptr = lil[pos].ptr;

      if (fIncr)
        lil[pos].setLinkCnt(lil[pos].getLinkCnt()+1);
      else
      {
        unsigned int uCnt = lil[pos].getLinkCnt();
        if (uCnt == 0)
        {
          nRetCode |= eLiErrUnderflow;
          std::cerr << "link count underflow! item ID=" << lil[pos].getID() << "\n";
          // todo: itemLinkCountUnderflow aufrufen
        }
        else
        {
          lil[pos].setLinkCnt(uCnt-1);
        }
      }
    }
  }
  else
  {
    // Missing item.
    parent->itemMissing(uIDToCount, expectedType);
    nRetCode |= eLiErrLink;
    *ptr = (TreeItem*)0;
  }
  
  return(nRetCode);
}
/*}}}*/

void LinkInfo::clear()                                        /*{{{*/
{
  while (lil.size())
    lil.pop_back();
}
/*}}}*/

TreeItem* LinkInfo::getItemWithID(unsigned int uID)           /*{{{*/
{
  unsigned int pos = getIndexNear(uID);
    
  if (pos < lil.size() && lil[pos].getID() == uID)
    return(lil[pos].ptr);
  else
    return((TreeItem*)0);
}
/*}}}*/

unsigned int LinkInfo::getIDMax()                             /*{{{*/
{
  if (lil.size() == 0)
    return(0);
  else
    return(lil[lil.size()-1].getID());
}
/*}}}*/

unsigned int LinkInfo::getLinkCnt(unsigned int uID)           /*{{{*/
{
  unsigned int pos = getIndexNear(uID);
  
  if (pos < lil.size() && lil[pos].getID() == uID)
  {
    return(lil[pos].getLinkCnt());
  }
  else
    return(0);
}
/*}}}*/

int LinkInfo::removeFromList(unsigned int uID)                /*{{{*/
{
  if (uID == 0)
    return(0);
  
  unsigned int pos = getIndexNear(uID);
  
  if (pos < lil.size() && lil[pos].getID() == uID)
  {
    lil.erase(lil.begin()+pos);
    return(0);
  }
  else
  {
    return(eLiErrIDNotInList);
  }
}
/*}}}*/

LinkInfo::LinkInfo(TreeView* myparent)
{
  parent = myparent;
}
