/*
 * Copyright (C) 2005, 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef SEARCHDATA_H
# define SEARCHDATA_H

# include <string>
# include <vector>

# define SD_KEYWORDS  0x01
# define SD_COMMENT   0x02
# define SD_TITLE     0x04
# define SD_PATH      0x08
# define SD_GROUP     0x10

/**
 * Data needed to describe a search. On the toplevel an instance of this is
 * created and COPIED on way down. Every item removes matching words from its
 * copy before giving it to the next level below. Once no words are left, every 
 * item below matches search criteria.
 */
class SearchData
{
public:
  
  int nFlags;
  
  void setWords(std::string text);
  
  void removeMatchingWords(std::string text);
  
  inline bool match() { return( (words.size() == 0) ); };
  
  inline int getNrOfWords() { return( words.size() ); };
  
  void AddWords(std::string& text);
  
private:  
  /**
   * Words to search for.
   */
  std::vector<std::string>   words;
};
#endif
