/*
 * Copyright (C) 2005-2008, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TREEGROUPBASE_H
# define TREEGROUPBASE_H

# include "treeitem.h"
# include "treepicbase.h"
# include "treepicdata.h"
# include "searchdata.h"

class TreeView;

class TreeGroupBase : public TreeItem
{
public:
  
  /**
   * 
   */
  bool canExpand(bool fShowPics);
  
  /**
   * 
   */
  TreeGroupBase(TreeItem* prev, TreeGroupBase* parent);
  
  /**
   * Does this group show its children in the browser view?
   */
  bool fExpanded;
  
  /**
   * Pointer to my first child.
   */
  TreeItem* itemFirstChild;
  
  /**
   * 
   */
  virtual ~TreeGroupBase();
  
  /**
   *
   */
  virtual std::string getSomeText(eText whichText);
  
  /**
   * 
   */
  virtual int loadFromXML_all(SimpleXMLTransfer*    source,
                              LinkInfo*             links) = 0;
  
  /**
   * 
   */
  virtual int saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion) = 0;
  
  /**
   * 
   */
  int loadFromXML_children(SimpleXMLTransfer*      source,
                           LinkInfo*               links);
  
  /**
   * 
   */
  int saveToXML_children(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion);
  
  /**
   * 
   */
  virtual int saveToXML_properties(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion) = 0;
  
  /**
   *
   */
  int deleteChildren();
  
  /**
   * 
   */
  enum eItemType getType();
  
  /**
   * 
   */
  virtual enum eItemLinkType getLinkType() = 0;
  
  /**
   * Returns a pointer to a TreePicBase, if one can be found.
   */
  TreePicBase* firstPicture();
  
  /**
   * Unmark all of my children. Is not recursive.
   */
  void unmarkAll();
  
  /**
   * Set/Reset mark recursive
   */
  void setMarkRec(bool fVal);
  
  /**
   * 
   */
  virtual void setTitle(const char* newtitle) = 0;
  
  /**
   * Connect links.
   */
  virtual int connectLinks(LinkInfo* links);
  
  /**
   * add a new group
   */
  TreeGroupBase* addNewGroup(const char*   name, 
                             eItemLinkType linktype);
  
  /**
   * add a new picture
   */
  TreePicBase*   addNewPic(eItemLinkType linktype);
  
  /**
   * returns pointer to last child
   */
  TreeItem* getLastChild();
  
  /**
   * 
   */
  virtual void changeBasepath(std::string oldbp, std::string newbp);
  
  /**
   * 
   */
  void addPics(std::string                    srcdir,
               std::string                    bp,
               TreePicData::eDateTimeSource   eDTS);
  
  /**
   * 
   */
  void addFile(std::string                    srcdir,
               std::string                    bp,
               TreePicData::eDateTimeSource   eDTS);
  
  
  /**
   * Takes item and links
   * it into this group after dest. If dest is 0 or not found in me,
   * it becomes the first child.
   * It returns a pointer to the new item.
   */
  TreeItem* linkAfter(TreeItem*    item, 
                      TreeItem*    dest,
                      unsigned int uLinkID);
  
  /**
   * Takes item out of its sourcegroup if its parent is set and inserts
   * it into this group after dest. If dest is 0 or not found in me,
   * it becomes the first child.
   */
  void moveAfter(TreeItem* item, TreeItem* dest);
  
  /**
   * Removes item out of my list.
   */
  void removeItem(TreeItem* item);
  
  /**
   * min and max have to be reset before calling this method!
   * It returns the number of non-groups.
   * Hidden items are not looked at.
   */
  unsigned int getMinMaxDateTime(JWWDateTime& min,
                                 JWWDateTime& max,
                                 bool         fRecursive);
  
  /**
   * 
   */
  void sortChildrenChrono();
  
  /**
   * 
   */
  void sortChildrenPath();
  
  /**
   * 
   */
  void sortChildrenSize(std::string bp);
  
  /**
   * 
   */
  virtual JWWDateTime getDT();
  
  /**
   * returns the number of missing files -- 0 if everything is fine
   */
  virtual unsigned int countMissingFiles(std::string basepath);
  
  /**
   * 
   */
  std::string findFile(std::string basepath, std::string filename);
  
  /**
   * Link every TreePicBase* inside and below myself into dest.
   * This can only be used for searches, as links don't create IDs.
   */
  void linkPicsInto(TreeGroupBase* dest);
  
  /**
   * Link marked items from src into myself, recursively.
   */
  void linkMarkedItems(TreeGroupBase* src);
  
  /**
   * Compares children (one and the next) and deletes duplicates.
   * Therefore they have to be sorted (for example chronologically)
   * to really delete duplicates.
   * Does only work on TreePicLink.
   */
  void deleteDuplicates();
  
  /**
   * 
   */
  void subdivideChrono(int level = 0);
  
  /**
   * Sets the user mark (fMark in TreeItem) on items matching the 
   * search criteria.
   * When creating a new view, DataToMark should be set to null.
   * If DataToMark is not null, "real" marking is done.
   */
  bool markMatchingItems(SearchData sd, TreeView* DataToMark = 0);
  
  /**
   * 
   */
  bool markItemsWithoutText();
  
  /**
   * 
   */
  void collapsePicsHidden();
  
  /**
   * Adds keywords to the list (no dupes, alphabetically)
   */
  void kw_setGroup(std::set<std::string>& kwset);
  
  virtual void kw_replaceGroup(std::string from, std::string to, int mode);
  
  /**
   * Count pictures inside of/including below me.
   */
  void countNonGroupItems(unsigned int& inside, unsigned int& insideandbelow);
  
  /**
   * Fills set recursively (no duplicates).
   */
  virtual void RecursiveList(std::set<TreeItem*>& set);
  
private:
  
  /**
   * Dieser Wert
   * zählt einfach die Anzahl der Gruppen in der Kinderliste und kann
   * nur über eine Methode abgefragt werden. Das @ref TreeGroupBase ist für den
   * korrekten Zustand verantwortlich. Von diesem Wert hängt ab, ob bei
   * (@ref fExpanded == false) ein 'Du kannst mich aufklappen-Indikator' angezeigt
   * wird. Nein, nicht nur von diesem Wert, sondern auch von der Tatsache, ob
   * Bilder im Baum angezeigt werden oder nicht und ob es überhaupt Kinder gibt.
   */
  unsigned int nGroups;   
};
#endif
