/*
 * Copyright (C) 2005, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treegroupdata.h"

#include "xmldefs.h"

std::string TreeGroupData::getText()
{
  return(title);
}

std::string TreeGroupData::getComment()
{
  return(comment);
}

std::string TreeGroupData::getKeywords()
{
  return(keywords);
}

TreeGroupData::TreeGroupData(TreeItem* prev, TreeGroupBase* parent) : TreeGroupBase(prev, parent)
{
  title    = "";
  keywords = "";
  comment  = "";
}

TreeGroupData::~TreeGroupData()
{
  deleteChildren();
}

int TreeGroupData::loadFromXML_all(SimpleXMLTransfer*    source,
                                   LinkInfo*             links)
{ 
  // Load my properties
  {
    TreeItem::loadFromXML_all(source, links);
    
    title    = source->attribute(XMLNAME_TITLE,    "");
    keywords = source->attribute(XMLNAME_KEYWORDS, "");
    comment  = source->attribute(XMLNAME_COMMENT,  "");
  }  
  
  loadFromXML_children(source, links);
  
  return(0);
}

int TreeGroupData::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  saveToXML_properties(dest, fConvLinks, nVersion);
  saveToXML_children(dest, fConvLinks, nVersion);
  
  return(0);
}

int TreeGroupData::saveToXML_properties(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  TreeItem::saveToXML_all(dest, fConvLinks, nVersion);

  dest->setName(XMLNAME_SAVE_ITEM);
  dest->addAttribute(XMLNAME_SAVE_TYPE,     XMLNAME_SAVE_TYPE_GROUP);
  dest->addAttribute(XMLNAME_SAVE_TITLE,    title);
  if (!keywords.empty())
    dest->addAttribute(XMLNAME_SAVE_KEYWORDS, keywords);
  
  if (!comment.empty())
  {
    if (nVersion != 0)
      comment = ConvertComment(comment, nVersion);
    
    dest->addAttribute(XMLNAME_SAVE_COMMENT, comment);
  }
  
  return(0);
}

enum TreeItem::eItemLinkType TreeGroupData::getLinkType()
{
  return(DATA);
}

void TreeGroupData::setTitle(const char* newtitle)
{
  title = newtitle;
}

void TreeGroupData::setKeywords(const char* nK)
{
  keywords = nK;
}

void TreeGroupData::setComment(const char* nC)
{
  comment = nC;
}

int  TreeGroupData::connectLinks(LinkInfo* links)
{
  int nRetCode = 0;
  
  // --- count my own links -----------------------------------------
  nRetCode |= TreeItem::countCommentLinks(links);
  
  // --- call children ----------------------------------------------
  nRetCode |= TreeGroupBase::connectLinks(links);
  
  return(nRetCode);
}
