/*
 * Copyright (C) 2005, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treegrouplinkproperties.h"

#include "xmldefs.h"
#include "treeview.h"

std::string TreeGroupLinkProperties::getText()
{
  if (linkptr != (TreeGroupBase*)0)
    return(linkptr->getText());
  else
    return("[error: link destination not found]");
}
  
std::string TreeGroupLinkProperties::getComment()
{
  if (linkptr != (TreeGroupBase*)0)
    return(linkptr->getComment());
  else
    return("[error: link destination not found]");
}  

std::string TreeGroupLinkProperties::getKeywords()
{
  if (linkptr != (TreeGroupBase*)0)
    return(linkptr->getKeywords());
  else
    return("");
}  

TreeGroupLinkProperties::TreeGroupLinkProperties(TreeItem* prev, TreeGroupBase* parent) : TreeGroupBase(prev, parent)
{
  linkptr = (TreeGroupBase*)0;
}

TreeGroupLinkProperties::TreeGroupLinkProperties(TreeItem* prev, TreeGroupBase* parent, TreeItem* dest) : TreeGroupBase(prev, parent)
{
  linkptr = (TreeGroupBase*)dest;
  linkID  = dest->getID();
}

TreeGroupLinkProperties::~TreeGroupLinkProperties()
{
  deleteChildren();
}

int TreeGroupLinkProperties::loadFromXML_all(SimpleXMLTransfer*   source,
                                             LinkInfo*            links)
{
  // Load my properties
  {
    linkID = source->attributeAsInt(XMLNAME_LINK);
    TreeItem::loadFromXML_all(source, links);
  }  

  loadFromXML_children(source, links);
  
  return(0);
}
   
int TreeGroupLinkProperties::saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  dest->setName(XMLNAME_SAVE_ITEM);
  if (fConvLinks)
  {
    std::string title    = getText();
    std::string keywords = getKeywords();
    std::string comment  = getComment();
    
    // I am a link in a 'virtual view' like a search result.
    dest->addAttribute(XMLNAME_SAVE_TYPE,     XMLNAME_SAVE_TYPE_GROUP);
    dest->addAttribute(XMLNAME_SAVE_TITLE,    title);
    if (!keywords.empty())
      dest->addAttribute(XMLNAME_SAVE_KEYWORDS, keywords);
    if (!comment.empty())
      dest->addAttribute(XMLNAME_SAVE_COMMENT,  comment);
  }
  else
  {
    // I am a real link
    dest->addAttribute(XMLNAME_SAVE_TYPE, XMLNAME_SAVE_TYPE_GROUPLINKPROPERTIES);
    saveToXML_properties(dest, fConvLinks, nVersion);
  }
  
  saveToXML_children(dest, fConvLinks, nVersion);
    
  return(0);
}

int TreeGroupLinkProperties::saveToXML_properties(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion)
{
  dest->addAttribute(XMLNAME_SAVE_LINK, linkID);
  TreeItem::saveToXML_all(dest, fConvLinks, nVersion);
  
  return(0);
}

enum TreeItem::eItemLinkType TreeGroupLinkProperties::getLinkType()
{
  return(LINKPROPERTIES);
}

void TreeGroupLinkProperties::setTitle(const char* newtitle)
{
  if (linkptr != (TreeGroupBase*)0)
    linkptr->setTitle(newtitle);  
}

void TreeGroupLinkProperties::setComment(const char* nC)
{
  if (linkptr != (TreeGroupBase*)0)
    linkptr->setComment(nC);  
}

void TreeGroupLinkProperties::setKeywords(const char* nK)
{
  if (linkptr != (TreeGroupBase*)0)
    linkptr->setKeywords(nK);
}

int TreeGroupLinkProperties::connectLinks(LinkInfo* links)
{
  int nRetCode = 0;
  
  // My comment is not a original one, so its links are not counted.
  
  // --- connect my own link ----------------------------------------  
  nRetCode |= links->countLinkCount((TreeItem**)(&linkptr), linkID, TreeItem::GROUP, true);
  
  // --- call children ----------------------------------------------
  nRetCode |= TreeGroupBase::connectLinks(links);
  
  return(nRetCode);
}
