/*
 * Copyright (C) 2005, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TREEGROUPLINKPROPERTIES
#define TREEGROUPLINKPROPERTIES

#include "treegroupbase.h"
#include "links.h"

/**
 * Link to a group. It links to a group's properties, 
 * but has its own children. It is used to hold the search
 * result of a <code>TreeGroupData</code>, but can also be used
 * in real data (for example 'best of some group').
 */
class TreeGroupLinkProperties : public TreeGroupBase
{
  public:
   
   /**
    *
    */
   virtual std::string getText();

   /**
    *
    */
   virtual std::string getComment();

   /**
    *
    */
   virtual std::string getKeywords();

   /**
    * 
    */
   TreeGroupLinkProperties(TreeItem* prev, TreeGroupBase* parent);
   
   /**
    * 
    */
   TreeGroupLinkProperties(TreeItem* prev, TreeGroupBase* parent, TreeItem* dest);
   
   /**
    * 
    */
   virtual ~TreeGroupLinkProperties();
   
   /**
    * 
    */
   virtual unsigned int getLinkID() { return(linkID); };
   
   /**
    * 
    */
   virtual TreeItem* getLinkPtr() { return((TreeItem*)linkptr); };
   
   /**
    * 
    */
   virtual int loadFromXML_all(SimpleXMLTransfer*    source,
                               LinkInfo*             links);
   
   /**
    * 
    */
   virtual int saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion);
      
   /**
    * 
    */
   virtual int saveToXML_properties(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion);

   /**
    * 
    */
   virtual enum eItemLinkType getLinkType();
      
   /**
    * 
    */
   virtual void setTitle(const char* newtitle);

   /**
    * 
    */
   virtual void setKeywords(const char* nK);

   /**
    * 
    */
   virtual void setComment(const char* nC);

   /**
    * Connect links.
    */
   virtual int connectLinks(LinkInfo* links);

  private:
   /**
    * The real group to which I link.
    */
   TreeGroupBase* linkptr;
   
   /**
    * The real group to which I link. In 'virtual views' like chronological views
    * or search results this is zero and only linkptr is valid!
    */
   unsigned int linkID;
};
#endif
