/*
 * Copyright (C) 2005, 2007, 2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "treepicbase.h"

#include <stdlib.h>
#include <iostream>
#include <sys/types.h> 
#include <sys/stat.h> 
#include <cstring>

#include "../config.h"
#include "../shared/platform.h"

#include "treegroupbase.h"
#include "treeview.h"
#include "../shared/lib_conversions.h"
#include "../shared/filetools.h"

enum TreePicBase::T_eRotMethod TreePicBase::eRotMethod = eRotUnknown;

TreePicBase::TreePicBase(TreeItem* prev, TreeGroupBase* parent) : TreeItem(prev, parent)
{
}

TreePicBase::~TreePicBase()
{
}

enum TreeItem::eItemType TreePicBase::getType()
{
  return(PIC);
}

int TreePicBase::editGimp(std::string bp)
{
  std::string tmp;
  
#ifdef WIN32
  tmp = "gimp " + bp + getPath();
#else  
  if (system("which gimp-remote") == 0)
    tmp = "gimp-remote " + bp + getPath();
  else if (system("which gimp") == 0)
    tmp = "gimp " + bp + getPath();
  else
    tmp = "defaultpaint " + bp + getPath(); // puppy linux
#endif  
  // todo: does only work if gimp is open
  // In the java version on woody I used 
  //     gimp-remote -n filename
  // which opened a new gimp if needed. Now -n is not 
  // documented anymore!
  std::cout << "command line: " << tmp << "\n";
  return(system(tmp.c_str()));
}
      
bool TreePicBase::markIfMatch(SearchData sd, TreeView* DataToMark)
{
  if (isHidden())
  {
    fMarked = false;
  }
  else
  {
    if ((sd.nFlags & SD_KEYWORDS) != 0)
    {
      sd.removeMatchingWords(getKeywords());
    }
    if ((sd.nFlags & SD_COMMENT) != 0)
    {
      sd.removeMatchingWords(getComment());
    }
    if ((sd.nFlags & SD_PATH) != 0)
    {
      sd.removeMatchingWords(getText());
    }

    
    bool fTmp = sd.match();
    if (fTmp && DataToMark)
      DataToMark->setMarkItem(this);
    fMarked = fTmp;
  }
  
  return(fMarked);
}

bool TreePicBase::markIfWithoutText()
{
  fMarked = (getKeywords().length() == 0) && (getComment().length() == 0);
  
  return(fMarked);
}

int TreePicBase::GetFileSize(std::string bp)
{
  return(FileTools::getSize(bp + "/" + getPath()));
}
