/*
 * Copyright (C) 2005, 2008, 2010  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#ifndef TREEPICDATA_H
# define TREEPICDATA_H

# include <string>

# include "treepicbase.h"
# include "datetime.h"

class TreePicData : public TreePicBase
{
public:
  
  /**
   * eEXIF:          1. exif  2. date = none, time = none
   * eEXIFFILE:      1. exif  2. file
   * eEXIFFILEDATE:  1. exif  2. date = file, time = none
   * eFILE:          1. file
   */
  enum eDateTimeSource { eEXIF = 0, eEXIFFILE = 1, eEXIFFILEDATE = 2, eFILE = 3, eABORT = 100 };
  
  /**
   * number of levels for 
   * std::string TreePicData::getDescr()
   */
  enum { eDescrLevels = 6 };
  
public:
  
  /**
   *
   */
  virtual std::string getText();
  
  /**
   *
   */
  virtual std::string getSomeText(eText whichText);
  
  TreePicData(TreeItem* prev, TreeGroupBase* parent);
  
  /**
   * Has to be called after data has been loaded.
   */
  void setHidden();
  
  /**
   * 
   */
  virtual ~TreePicData();
  
  /**
   * 
   */
  virtual int loadFromXML_all(SimpleXMLTransfer*    source,
                              LinkInfo*             links);
  
  /**
   * 
   */
  virtual int saveToXML_all(SimpleXMLTransfer* dest, bool fConvLinks, int nVersion);
  
  /**
   * 
   */
  virtual enum eItemLinkType getLinkType();
  
  /*
   * 
   */
  virtual std::string getPath();
  
  /**
   * 
   */
  virtual std::string getComment();
  
  /**
   * 
   */
  virtual std::string getKeywords();
  
  /**
   * 
   */
  virtual std::string getTimeString();
  
  /**
   * 
   */
  virtual std::string getDateString();
  
  /**
   * 
   */
  virtual unsigned int getYear();
  
  /**
   * 
   */
  virtual bool hasTime();
  
  /**
   * 
   */
  virtual bool hasDate();
  
  /**
   * 
   */
  virtual JWWDateTime getDateTime();
  
  /**
   * 
   */
  virtual std::string getDescr(int level, bool fShort=false);
  
  /**
   * Connect and/or count links.
   */
  virtual int connectLinks(LinkInfo* links);
  /**
   * 
   */
  virtual void setKeywords(const char* nK);
  
  /**
   * 
   */
  virtual void setComment(const char* nC);
  
  /**
   * Setzt das Datum aus einem String der Form YYYY-MM-DD
   */
  virtual int setDate(const char* nD);
  
  /**
   * Setzt die Zeit aus einem String der Form HH:MM:SS
   */
  virtual int setTime(const char* nT);
  
  /**
   * 
   */
  virtual void changeBasepath(std::string oldbp, std::string newbp);
  
  /**
   * 
   */
  void setData(std::string       newpath, 
               std::string       basepath,
               eDateTimeSource   eDTS);
  
  /**
   * 
   */
  virtual JWWDateTime getDT();
  
  /**
   * Returns zero if the linked file exists, one otherwise.
   */
  virtual unsigned int countMissingFiles(std::string basepath);
  
  /**
   * 
   */
  void timeshift(int nSec);
  
  /**
   * Returns true if this picture is hidden because of something
   */
  virtual bool isHidden();

  void ImportMetadata(const std::string basepath);
  
private:
  
  /**
   * 
   */
  JWWDateTime dt;
  
  /**
   * 
   */
  std::string path;
  
  /**
   * 
   */
  std::string keywords;
  
  /**
   * 
   */
  std::string comment;
  
  /**
   * 
   */
  bool fHidden;
  
  /**
   * returns base path
   */
  std::string getBasepath();      
};
#endif

