// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2011  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

// Timing-Ergebnisse auf netbook mit SXT@1:
// timing.load=404
// timing.save=183
// timing.load=406
// timing.compare=31
// 
// Timing-Ergebnisse auf netbook mit SXT@2:
// timing.load=258
// 
// Timing-Ergebnisse auf X20 mit SXT@1:
// timing.load=230
// timing.save=130
// timing.compare=37
// 
// Timing-Ergebnisse auf X20 mit SXT@2:
// timing.load=161
// 
// Neuerdings X20 mit SXT@3:
// timing.load=332
// timing.save=248
// timing.load=337
// timing.compare=83
// 
// Neuerdings X20 mit SXT@1:
// timing.load=496
// timing.save=258
// timing.load=508
// timing.compare=85
// 
// Die Verbesserung ist nach wie vor vorhanden aber beide 
// Varianten nur halb so schnell wie vorher? Langsamer als das Netbook!
// Wird nicht mehr schnell genug von 800MHz auf 1,6GHz umgeschaltet? Aber am
// System habe ich doch nix verändert...
// 
// X20 mit SXT@3 -- neu gebootet, die 8GB-SDHC steckt nicht mehr:
// timing.load=146
// timing.save=116
// timing.load=148
// timing.compare=36



#include "treeview.h"

#include "../shared/SimpleXMLTransfer.h"
#include "../shared/filetools.h"
#include "../shared/lib_conversions.h"
#include "../shared/hirestime.h"
#include "treegroupbase.h"
#include "treekwhandler.h"

#include <iostream>
#include <fstream>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/stat.h>

#define FILE_VERSION_CURRENT 3

const unsigned int TreeView::uThumbSizes[] = {50, 100, 150, 200, 250, 300, 450, 0};

TreeView::TreeView(TreeView* mysource)                          
{
  std::cout << "------------------------------------------------\n";
  std::cout << "TreeView(" << this << ")::TreeView()\n";
  
  // Indicate that @ref root is empty.
  root   = (TreeGroupRoot*)0;
  source = mysource;
  kw     = new TreeKwHandler(this);
  perm   = new T_Perm();
  
  HiddenKeywords.clear();
  
  if (source != (TreeView*)0)
  {
    basePath = source->getBasepath();
  }
  
  //
  linkInfo = new LinkInfo(this);
  
  //
  groupWithMarkedItems = (TreeGroupBase*)0;
  uMarkedItems         = 0;
  fAddDelPicAllowed    = true;
  
  projectfilePath = "";
  strAddPath = "";
  createRoot();
  std::cout << "------------------------------------------------\n";
}


TreeView::~TreeView()                                           
{
  std::cout << "------------------------------------------------\n";
  std::cout << "TreeView(" << this << ")::~TreeView()\n";
  
  deleteRoot();

  delete kw;
  delete linkInfo;
  delete perm;
  
  std::cout << "------------------------------------------------\n";  
}


void TreeView::createRoot()                                     
{
  if (root == (TreeGroupRoot*)0)
  {
    root = new TreeGroupRoot((TreeItem*)0, (TreeGroupBase*)0, this);
    root->SetVersionInfo(FILE_VERSION_CURRENT);
  }
  // todo: title is language dependent
  root->setTitle("Root");
}


void TreeView::deleteRoot()                                     
{
  // Make sure @ref root is empty.
  if (root != (TreeGroupRoot*)0)
  {
    delete root;
    root = (TreeGroupRoot*)0;
  }
}


int TreeView::loadFromFile(std::string newProjectfilePath)      
{
  int retCode = eLE_NO_ERROR;

  if (FileTools::fileExists(newProjectfilePath) == false)
    return(eLE_NOFILE);
  
  std::cout << "------------------------------------------------------------------------------------\n";
  std::cout << "int TreeView::loadFromFile(std::string newProjectfilePath = " << newProjectfilePath << ")\n";
  
  deleteRoot();
  while (basePaths.size())
    basePaths.pop_back();
  fAddDelPicAllowed = false;
  
  linkInfo->clear();
  
  // Load data into a SimpleXMLTransfer and copy to internal structure afterwards.
  // The internal classes could be enabled to read a file by themselves, but it this
  // case I'd have to maintain more than one method of reading/writing xml.
  std::cout << "Loading " << newProjectfilePath << " into SimpleXMLTransfer\n";
  SimpleXMLTransfer* data;
  try
  {
    HiResTime hrt;
    hrt.Capture();
    
    data = new SimpleXMLTransfer(newProjectfilePath);
    std::cout << "timing.load=" << hrt.GetMSecSinceCapture() << "\n";
  }
  catch (XMLException e)
  {
    std::cout << "Exception when loading datafile: " << e.what() << "\n";
    return(eLE_XML);
  }
  
  if (data->getInt("version", 2) > FILE_VERSION_CURRENT)
  {
    return(eLE_VERSION);    
  }
  
  root = new TreeGroupRoot((TreeItem*)0, (TreeGroupBase*)0, this);

  // 
  if (data->indexOfChild("hidden") >= 0)
  {
    SimpleXMLTransfer* hidden = data->getChild("hidden");
    SimpleXMLTransfer* i;
    unsigned int size = hidden->getChildCount();
    
    std::cout << "Hidden:\n";
    for (unsigned int cnt=0; cnt<size; cnt++)
    {
      i = hidden->getChildAt(cnt);
      // std::cout << "  " << i->getString("keyword") << "\n";
      HiddenKeywords.insert(strU(i->getString("keyword")));
    }    
  }
  
  perm->SetPermissions(data->getInt("permission.this",    0x7FFFFFFF),
                       data->getInt("permission.save_as", 0x7FFFFFFF));
  
  // 
  if (data->getChild("config")->indexOfChild("thumbdir") >= 0)
  {
    SimpleXMLTransfer* hidden = data->getChild("config.thumbdir");
    
    thumbdir = (eThumbDir)(hidden->attributeAsInt("type"));
    thumbdirselect = hidden->attribute("dir");
  }
  else
  {
    thumbdir       = eHome; // save default to not clutter someones data directory
    thumbdirselect = "";
  }

  strBasepathRelOriginal = data->getString("config.basePathRel");
  
  std::cout << "Copying SimpleXMLTransfer\n";
  std::cout << "Create Table: ID -> TreeItem*\n";
  root->SetVersionInfo(data->attributeAsInt("version", 2));
  root->loadFromXML_all(data->getChild("data"), linkInfo);
  
  std::cout << "Connect links\n";
  if (root->connectLinks(linkInfo))
  {
    retCode |= (int)(eLE_LINK);
  }

  showLinkableItems();

  std::cout << "Basepath?\n";
  {
    std::string  basepath;

    basepath = data->getString("config.basePath");
    basePaths.push_back(basepath);
    
    if (data->getChild("config")->indexOfChild("alternativeBasePaths") >= 0)
    {
      SimpleXMLTransfer* group = data->getChild("config.alternativeBasePaths");      
      int nSize = group->getChildCount();
      for (int n=0; n<nSize; n++)
      {
        basepath = group->getChildAt(n)->attribute("string");
        basePaths.push_back(basepath);
      }
    }
    
    TreePicBase* testpic = root->firstPicture();
    
    if (testpic == (TreePicBase*)0)
    {
      basePath          = basePaths[0];
      fAddDelPicAllowed = true;
    }
    else
    {
      std::string testpath = testpic->getPath();
      if (FileTools::fileExists(basePaths[0] + testpath))
      {
        basePath          = basePaths[0];
        fAddDelPicAllowed = true;
      }
      else
      {
        // relative path?
        basePath = FileTools::createAbsPath(FileTools::path(newProjectfilePath),
                                            strBasepathRelOriginal);
        
        if (FileTools::fileExists(basePath + testpath) == false)
        {
          basePath = "";
          
          for (unsigned int n=1; n<basePaths.size() && basePath.length() == 0; n++)
          {
            if (FileTools::fileExists(basePaths[n] + testpath))
              basePath = basePaths[n];            
          }
          
          if (basePath.length() == 0)
          {
            basePath  = basePaths[0];
            retCode  |= (int)(eLE_BASEPATH);
          }
        }  
        else
        {
          // Apparently, album file and data directory have been moved. Update
          // primary basepath accordingly.
          basePaths[0] = basePath;
        }
      }
    }    
  }

  strAddPath = data->getString("config.addPath", basePath);
  
  std::cout << "Deleting SimpleXMLTransfer\n";
  delete data;

  createRoot();
  
  std::cout << "------------------------------------------------------------------------------------\n";

  projectfilePath = newProjectfilePath;
  
  return(retCode);
}

SimpleXMLTransfer* TreeView::saveToSXT(std::string newProjectfilePath, bool fConvLinks, int nVersion)
{
  SimpleXMLTransfer* project;
  
  project = new SimpleXMLTransfer();
  project->setName("album");
  project->addAttribute("version", nVersion);

  // permission info
  {
    SimpleXMLTransfer* xPerm;
    xPerm = new SimpleXMLTransfer();
    xPerm->setName("permission");
    xPerm->addAttribute("this",    perm->uPermMask);
    xPerm->addAttribute("save_as", perm->uPermMaskSaveAs);
    project->addChild(xPerm);
  }
  
  // config info
  SimpleXMLTransfer* config;
  config = new SimpleXMLTransfer();
  config->setName("config");
  {
    std::string bp = "";
    
    if (basePaths.size())
      bp = basePaths[0];
    else if (source != 0)
      bp = source->getBasepath();
  
    config->addAttribute("basePath",    bp);
    
    if (perm->IsPermitted(PERM_OverwriteAlbum))
    {
      // todo: das funktioniert nicht korrekt bei win32-Pfaden?
      config->addAttribute("basePathRel", FileTools::createRelPath(newProjectfilePath, bp));
    }
    else
      config->addAttribute("basePathRel", strBasepathRelOriginal);
    
    config->addAttribute("addPath", strAddPath);
  }
  
  if (HiddenKeywords.size())
  {
    SimpleXMLTransfer*              hidden;
    SimpleXMLTransfer*              tmp;
    std::set<std::string>::iterator i = HiddenKeywords.begin();
    
    hidden = new SimpleXMLTransfer();
    hidden->setName("hidden");
        
    while (i != HiddenKeywords.end())
    {
      tmp =  new SimpleXMLTransfer();
      tmp->setName("keyword");
      tmp->addAttribute("keyword", *i);
      hidden->addChild(tmp);
      
      i++;
    }     
    project->addChild(hidden);
  }
  
  //
  {
    config->setAttribute("thumbdir.type", thumbdir);
    config->setAttribute("thumbdir.dir",  thumbdirselect);      
  }
  
  {
    SimpleXMLTransfer* alt;
    SimpleXMLTransfer* abp;
    
    alt = new SimpleXMLTransfer();
    alt->setName("alternativeBasePaths");
    
    for (unsigned int n=1; n<basePaths.size(); n++)
    {
      abp = new SimpleXMLTransfer();
      abp->setName("basePath");
      abp->addAttribute("string", basePaths[n]);
      alt->addChild(abp);
    }
    config->addChild(alt);
  }
  project->addChild(config);
  
  // Copy internal data to a SimpleXMLTransfer
  SimpleXMLTransfer* data;
  data    = new SimpleXMLTransfer();
  
  // Set target version to 0 in order to minimize checks in case
  // no conversion is needed:
  if (nVersion != root->GetVersionInfo())
    root->saveToXML_all(data, fConvLinks, nVersion);
  else
    root->saveToXML_all(data, fConvLinks, 0);
      
  data->setName("data");
  project->addChild(data);
  
  return(project);
}

int TreeView::saveAsNewProject(std::string newProjectfilePath)
{
  bool fVirtualView = (source != (TreeView*)0);
  
  std::ofstream outfile;
  
  outfile.open(newProjectfilePath.c_str());
  if (!outfile)
    return(-2);

  // change permission if the file is "saved as"
  if (newProjectfilePath.compare(projectfilePath) != 0)
    perm->SetPermissions(perm->uPermMaskSaveAs, perm->uPermMaskSaveAs);
  
  SimpleXMLTransfer* project = saveToSXT(newProjectfilePath, fVirtualView, FILE_VERSION_CURRENT);
  root->SetVersionInfo(FILE_VERSION_CURRENT);
  
  HiResTime hrt;
  hrt.Capture();
  // Save to file: not using indentation to make file smaller, which also
  // speeds up loading and saving.
  project->print(outfile, 0, 0, 0);
  outfile.close();
  std::cout << "timing.savetofile=" << hrt.GetMSecSinceCapture() << "\n";
  delete project;
  
  if (!fVirtualView)
    projectfilePath = newProjectfilePath;
  
  return(0);  
}


int TreeView::saveToProjectFile()                               
{
  if (source != (TreeView*)0)
  {
    return(source->saveToProjectFile());
  }
  else
  {
    return(saveAsNewProject(projectfilePath));
  }
}


TreeItem* TreeView::browserItemPrev(TreeItem* p, 
                                    bool      fShowPics)        
{
  TreeItem* prev = p->itemPrev;

  // skip hidden items
  while (prev != (TreeItem*)0 && prev->isHidden())
    prev = prev->itemPrev;
  
  if (prev == (TreeItem*)0)
  {
    // There is no previous item in this group. The previous item is 
    // the group itself.
    return(p->itemParent);
  }
  else
  {
    if (prev->getType() == TreeItem::PIC)
    {
      if (fShowPics)
        return(prev);
      // I have to skip pictures:
      prev = prev->itemPrev;
      while (prev != (TreeItem*)0 && prev->getType() == TreeItem::PIC)
        prev = prev->itemPrev;
    }
    // Now prev points to a group or to 0.
    if (prev == (TreeItem*)0)
    {
      // There is no previous item in this group. The previous item is 
      // the group itself.
      return(p->itemParent);
    }
    
    // Now I have to go down as far as possible, starting from the previous group.
    
    while (1)
    {
      // prev points to a group
      if (((TreeGroupBase*)prev)->fExpanded == false)
        return(prev);
      else
      {
        // The group is expanded. I need to go down as far as possible and return
        // that item. Because the group is expanded, it contains something to display, 
        // which means I won't reach 0 when going backwards starting from its last child.
        prev = ((TreeGroupBase*)prev)->itemFirstChild;
        while (prev->itemNext != (TreeItem*)0)
          prev = prev->itemNext;
        // Now prev points to the last item of the group.
        // 
        // skip hidden items
        while (prev != (TreeItem*)0 && prev->isHidden())
          prev = prev->itemPrev;
        
        if (prev->getType() == TreeItem::PIC)
        {
          if (fShowPics)
            return(prev);
          // I have to skip pictures:
          prev = prev->itemPrev;
          while (prev != (TreeItem*)0 && prev->getType() == TreeItem::PIC)
            prev = prev->itemPrev;
        }
      }
    }    
  }
}


TreeItem* TreeView::browserItemNext(TreeItem* p, 
                                    bool      fShowPics)        
{
  TreeItem* next;

  if (p == (TreeItem*)0)
    return(p);
  
  if (p->getType() == TreeItem::GROUP && ((TreeGroupBase*)p)->fExpanded == true)
  {
    // The group is expanded. So it does contain at least one item which can be displayed.
    p = ((TreeGroupBase*)p)->itemFirstChild;
    
    // There is an exception: root. It may be expanded without children.
    if (p == (TreeItem*)0)
      return(p);
    
    while (p != (TreeItem*)0 && p->getType() == TreeItem::PIC && (!fShowPics || p->isHidden()))
      p = p->itemNext;
    
    return(p);        
  }
  else
  {
    while (1)
    {
      next = p->itemNext;
      
      while (next != (TreeItem*)0 && next->getType() == TreeItem::PIC && next->isHidden())
        next = next->itemNext;
            
      if (next != (TreeItem*)0 && next->getType()==TreeItem::PIC)
      {
        if (fShowPics)
          return(next);

        next = next->itemNext;
        
        while (next != (TreeItem*)0 && next->getType() == TreeItem::PIC && (!fShowPics || next->isHidden()))
          next = next->itemNext;
      }
      // Now next points to a group or to 0.
      if (next != (TreeItem*)0)
        return(next);
    
      // I reached the end of a group.
      p = p->itemParent;
      if (p == (TreeItem*)0)
        return(p);
    }
  }    
}


void TreeView::toggleMarkItem(TreeItem* p)                      
{
  std::cout << "-----------------------------------------------------\n";
  std::cout << "void TreeView(" << this << ")::toggleMarkItem(TreeItem* p = " << p << ")\n";
  
  if (p->fMarked)
    resetMarkItem(p);
  else
    setMarkItem(p);
  std::cout << "-----------------------------------------------------\n";
}


void TreeView::setMarkItem(TreeItem* p)                         
{
  std::cout << "-----------------------------------------------------\n";
  std::cout << "void TreeView(" << this << ")::setMarkItem(TreeItem* p = " << p << ")\n";
  
  if (p->fMarked == false)
  {
    std::cout << "unmark? " << groupWithMarkedItems << " " << p->itemParent << "\n";
    if (groupWithMarkedItems != (TreeGroupBase*)0 &&
        groupWithMarkedItems != (TreeGroupBase*)(p->itemParent))
    {
      groupWithMarkedItems->unmarkAll();
      uMarkedItems = 0;
    }
    groupWithMarkedItems = (TreeGroupBase*)(p->itemParent);
    std::cout << "groupWithMarkedItems = (TreeGroupBase*)(p->itemParent) = " 
      << p->itemParent << " = " << groupWithMarkedItems << "\n";
    p->fMarked = true;
    uMarkedItems++;    
  }
  std::cout << "-----------------------------------------------------\n";
}


void TreeView::resetMarkItem(TreeItem* p)                       
{
  std::cout << "-----------------------------------------------------\n";
  std::cout << "void TreeView(" << this << ")::resetMarkItem(TreeItem* p = " << p << ")\n";
  
  if (p == (TreeItem*)0)
  {
    if (groupWithMarkedItems != (TreeGroupBase*)0)
    {
      groupWithMarkedItems->unmarkAll();
      
      groupWithMarkedItems = (TreeGroupBase*)0;
      uMarkedItems         = 0;
    }
  }
  else if (p->fMarked)
  {
    p->fMarked = false;
    uMarkedItems--;
    if (uMarkedItems == 0)
    {
      std::cout << "groupWithMarkedItems = (TreeGroupBase*)0;\n";
      groupWithMarkedItems = (TreeGroupBase*)0;
    }
  }
  std::cout << "-----------------------------------------------------\n";
}


std::string TreeView::getPicPath(TreeItem* item,
                                 int       nSize)               
{
  if (nSize)  
  {
    std::string picPath;
    eThumbDir   th;

    exchangeThumbSettings(th, picPath, false);
    
    switch (th)
    {
     case eSrc:
      picPath = basePath + item->getText();
      break;
     case eHome:
      picPath = FileTools::getHomeDir("Fotoalbum") + FileTools::stripAbsFromPath(basePath) + item->getText();
      break;
     default:
      picPath += "/" + FileTools::stripAbsFromPath(basePath) + item->getText();
      break;
    }
    
    std::string tmp        = picPath.substr(0, picPath.rfind('/')+1);
    std::string sizestring = itoStr(nSize, ' ', 0);
    
    tmp.append(".thumbnails/");
    tmp.append(sizestring);
    tmp.append("x");
    tmp.append(sizestring);
    tmp.append("/");
    tmp.append(picPath.substr(picPath.rfind('/')+1));
      
    // Sicherstellen, dass die Endung auf jpeg-Format deutet.
    std::string::size_type pos = picPath.rfind('.');
    if (pos != std::string::npos)
    {
      std::string endung = strU(picPath.substr(pos+1));
      
      if (endung.compare("JPG") != 0 && endung.compare("JPEG") != 0)
        tmp.append(".jpeg");
    }
    else
      tmp.append(".jpeg");
    
    return(tmp);
  }
  else
    return(basePath + item->getText());
}


bool TreeView::makeVisible(TreeItem* item)                      
{
  TreeGroupBase* group = (TreeGroupBase*)(item->itemParent);
  bool           fTmp  = false;
  
  while (group != (TreeGroupBase*)0)
  {
    if (!group->fExpanded)
    {
      group->fExpanded = true;
      fTmp             = true;
    }
    group = (TreeGroupBase*)(group->itemParent);
  }
  
  return(fTmp);
}


bool TreeView::hasData()                                        
{
  return(
         (root != (TreeGroupRoot*)0) && (root->itemFirstChild != (TreeItem*)0)
         );
}


TreeItem* TreeView::resolveInternalLink(std::string linktext)   
{
  if (linktext.find("<ID=") == 0)
  {
    LinkInfo* links = getLinkInfoList();

    return(links->getItemWithID(atoi(linktext.substr(4, linktext.find('>')-4).c_str())));
  }
  else
    return((TreeItem*)0);
}


std::string TreeView::resolveExternalLink(std::string linktext,
                                          TreeItem*   item)     
{
  if (linktext.find("<FILE=") == std::string::npos)
    return("");
  
  linktext = linktext.substr(6);
  linktext = linktext.substr(0, linktext.length()-1);
  
  std::cout << "linktext: *" << linktext << "*\n";
  
  if (item->getType() != TreeItem::GROUP)
    item = item->itemParent;
  
  return(((TreeGroupBase*)item)->findFile(basePath, linktext));
}


void TreeView::itemMissing(unsigned int        uID, 
                           TreeItem::eItemType type)            
{
  // todo: show this error (more verbose) to the user
  std::cerr << "Item missing. ID=" << uID << " type=" << type << "\n";
}


void TreeView::itemWrongType(unsigned int        uID,
                             TreeItem::eItemType type)          
{
  // todo: show this error (more verbose) to the user
  LinkInfo*           links = getLinkInfoList();
  TreeItem*           i     = links->getItemWithID(uID);
    
  std::cerr << "Item with wrong type. ID=" << uID << " type=" << type << ". Type found: " << i->getType() << "\n";
}


LinkInfo* TreeView::getLinkInfoList()                           
{
  if (source != (TreeView*)0)
    return(source->getLinkInfoList());
  else
    return(linkInfo);
}


unsigned int  TreeView::countMarkedItems()                      
{
  return(uMarkedItems);  
}


TreeItem::eItemType TreeView::markedItemsType()                 
{
  TreeItem::eItemType tmp = TreeItem::NONE;

  if (groupWithMarkedItems == (TreeGroupBase*)0)
    return(tmp);
  
  TreeItem* p = groupWithMarkedItems->itemFirstChild;
  
  while (p != (TreeItem*)0)
  {
    if (p->fMarked)
    {
      if (tmp == TreeItem::NONE)
        tmp = p->getType();
      else
      {
        if (p->getType() != tmp)
          return(TreeItem::ANY);
      }
    }
    p = p->itemNext;
  }
  
  return(tmp);
}


std::string TreeView::getProjectfilePath()                      
{
  return(projectfilePath);
}


unsigned int TreeView::getIDMax()                               
{
  LinkInfo* li = getLinkInfoList();
  
  return(li->getIDMax());  
}


void TreeView::newLinkableItem(TreeItem* item)                  
{
  getLinkInfoList()->newLinkableItem(item);
}


void TreeView::showLinkableItems()                              
{
  getLinkInfoList()->showList();
}


int  TreeView::addPicsFromDir(const char*                  picdir, 
                              TreeGroupBase*               lastPopupSingleItem,
                              TreePicData::eDateTimeSource eDTS)  
{
  int nRetCode = 0;
  
  std::string baseURL = picdir;
  
  if (baseURL.length() == 0)
    baseURL = "/";

  if (baseURL[baseURL.length()-1] != '/')
    baseURL += "/";      

  std::cout << "----------------------------------------------\n";
  std::cout << "Adding pictures from " << baseURL << "\n";

  updateBasepath(baseURL.c_str());

  lastPopupSingleItem->addPics(baseURL, basePath, eDTS);
  
  return(nRetCode);
}


int  TreeView::addFile(const char*                  picdir, 
                       TreeGroupBase*               lastPopupSingleItem,
                       TreePicData::eDateTimeSource eDTS)  
{
  int nRetCode = 0;
  
  std::string baseURL = FileTools::path(picdir);
    
  updateBasepath(baseURL.c_str());

  lastPopupSingleItem->addFile(picdir, basePath, eDTS);
  
  return(nRetCode);
}


int  TreeView::updateBasepath(const char* picdir)                 
{
  std::string baseURL = picdir;
  
  {
    strAddPath = picdir;
    std::string::size_type pos = strAddPath.rfind('/');
    if (pos == strAddPath.length()-1)
      pos = strAddPath.rfind('/', pos-1);
    if (pos != std::string::npos)
      strAddPath = strAddPath.substr(0, pos);
  }
  
  std::cout << "-----------------------------------------------\n";
  std::cout << "basepath old = " << basePath << "\n";
    
  // Wenn es basePath noch nicht gibt, kann ich ihn mal neu setzen.
  // Wie ich es jetzt auch immer mache, es wird falsch sein. Aber 
  // immerhin ist er mal gesetzt.
  if (basePaths.size() == 0)
  {
    basePaths.push_back(baseURL);
    basePath          = baseURL;
    fAddDelPicAllowed = true;
  }
            
  // Wenn der neue Pfad nicht unterhalb von basePath liegt, muss ich den
  // anpassen. 
  if (baseURL.find(basePath) != 0)
  {
    std::string newbp;
    
    // Maybe baseURL is the new basepath?
    if (basePath.find(baseURL) == 0)
    {
      // Yes! Easy...
      newbp = baseURL;
    }
    else
    {
      // Jetzt muss ich die gemeinsame Wurzel nehmen.
      // Um die gemeinsame Wurzel zu finden, schneidet man den einen 
      // so lange ab, bis er im anderen auftaucht:
      baseURL = baseURL.substr(0, baseURL.rfind('/'));      
      while (basePath.find(baseURL) == std::string::npos && baseURL.length() > 1)
      {
        baseURL = baseURL.substr(0, baseURL.rfind('/'));
      }
      
      newbp = baseURL + "/";
    }
    
    // Gut. 'baseURL' ist also der neue 'basePath'.
    // Die vorhandenen Bilder anpassen:
    root->changeBasepath(basePath, newbp);
    // Jetzt ändern:
    basePath     = newbp;
    basePaths[0] = basePath;
  }

  std::cout << "basepath new = " << basePath << "\n";
  std::cout << "-----------------------------------------------\n";  
  
  return(0);
}


void TreeView::linkInto_marked(TreeGroupBase* destgroup)          
{
  TreeItem*    p = groupWithMarkedItems->itemFirstChild;
  TreeItem*    t;
  TreeItem*    newItem;
  unsigned int uID;
  
  while (p != (TreeItem*)0)
  {     
    if (p->fMarked)
    {
      t = destgroup->getLastChild();
      
      // item has to be linkable:
      {
        uID = p->getID();
        if (uID == 0)
        {
          uID = getIDMax()+1;
          p->setID(uID);
          newLinkableItem(p);
        }
      }

      newItem = destgroup->linkAfter(p, t, uID);
      newItem->connectLinks(getLinkInfoList());
    }
    
    p = p->itemNext;
  }  
}


void TreeView::moveInto_marked(TreeGroupBase* destgroup)          
{
  TreeItem* p = groupWithMarkedItems->itemFirstChild;
  TreeItem* t;
  TreeItem* ptmp;
  
  while (p != (TreeItem*)0)
  { 
    // Setting to the next item has to be done before moving the current
    // item:
    ptmp = p->itemNext;
    
    if (p->fMarked)
    {
      t = destgroup->getLastChild();
      destgroup->moveAfter(p, t);
      // The item is not in the group groupWithMarkedItems anymore,
      // so it must not be marked:
      p->fMarked = false;
      uMarkedItems--;
      if (uMarkedItems == 0)
        groupWithMarkedItems = (TreeGroupBase*)0;
    }
    
    p = ptmp;
  }  
}


void TreeView::moveAfter_marked(TreeItem* destitem)               
{
  TreeItem*      p = groupWithMarkedItems->itemFirstChild;
  TreeItem*      ptmp;
  TreeGroupBase* destgroup = (TreeGroupBase*)(destitem->itemParent);
  
  while (p != (TreeItem*)0)
  { 
    // Setting to the next item has to be done before moving the current
    // item:
    ptmp = p->itemNext;
    
    if (p->fMarked)
    {
      destgroup->moveAfter(p, destitem);
      // change destitem so that the item stay in their previous order
      destitem = p;
      // The item is not in the group groupWithMarkedItems anymore,
      // so it must not be marked:
      p->fMarked = false;
      uMarkedItems--;
      if (uMarkedItems == 0)
        groupWithMarkedItems = (TreeGroupBase*)0;
    }
    
    p = ptmp;
  }  
}


std::string TreeView::getBasepath()                               
{
  return(basePath);
}


bool TreeView::isAddDelAllowed()                                  
{
  return(fAddDelPicAllowed);
}


int  TreeView::deleteItem(TreeItem* item, bool fDelFiles)         
{
  int nRetCode = 0;
  
  // groups: only allowed if empty
  if (item->getType() == TreeItem::GROUP && ((TreeGroupBase*)item)->itemFirstChild != (TreeItem*)0)
  {
    return(0x01);
  }
  
  LinkInfo* li = getLinkInfoList();  
  
  // only allowed if not pointed to
  {
    unsigned int uID  = item->getID();
    
    if (li->getLinkCnt(uID) > 0)
      return(0x02);
  }
      
  // handle mark
  if (item->fMarked)
    uMarkedItems--;
  if (uMarkedItems == 0)
    groupWithMarkedItems = (TreeGroupBase*)0;
  
  // remove files?
  if (fDelFiles && item->getType() == TreeItem::PIC && item->getLinkType() == TreeItem::DATA)
  {    
    // remove file
    FileTools::rm(getPicPath(item, 0));
  
    // remove thumbnails
    for (unsigned int n=0; uThumbSizes[n] > 0; n++)
    {
      std::cout << "trying to remove " << uThumbSizes[n] << "\n";
      FileTools::rm(getPicPath(item, uThumbSizes[n]));
    }
  }
  
  // remove from tree
  ((TreeGroupBase*)item->itemParent)->removeItem(item);

  // decrement link count
  if (item->getLinkType() == TreeItem::DATA)
  {
    nRetCode |= item->countCommentLinks(li, false);
  }

  {
    TreeItem* dummy;
    nRetCode |= li->countLinkCount(&dummy, item->getLinkID(), TreeItem::ANY, false);
  }

  // remove from linkInfoList, if it is in it
  nRetCode |= li->removeFromList(item->getID());
  
  // remove from memory
  delete item;
  
  return(nRetCode);
}


bool TreeView::dataSaved()                                        
{
  bool               fSaved;
  
  HiResTime hrt;
  hrt.Capture();
    
  // Save with current version in order to not apply file format updates (would result
  // in file changes although the user didn't change anything).  
  SimpleXMLTransfer* data_new = saveToSXT(projectfilePath, false, root->GetVersionInfo());
  
  std::cout << "timing.save=" << hrt.GetMSecSinceCapture() << "\n";
  hrt.Capture();
    
  SimpleXMLTransfer* data_old = new SimpleXMLTransfer(projectfilePath);
  std::cout << "timing.load=" << hrt.GetMSecSinceCapture() << "\n";
  
  // remove variable entries
  {
    SimpleXMLTransfer* tmp;
    int idx = data_new->indexOfChild("permission");
    if (idx >= 0)
    {
      tmp = data_new->getChildAt(idx);
      data_new->removeChildAt(idx);
      delete tmp;
    }
    idx = data_old->indexOfChild("permission");
    if (idx >= 0)
    {
      tmp = data_old->getChildAt(idx);
      data_old->removeChildAt(idx);
      delete tmp;
    }
  }
  
  hrt.Capture();
  fSaved = data_old->equalsOrdered(data_new);
  std::cout << "timing.compare=" << hrt.GetMSecSinceCapture() << "\n";
  
  delete data_new;
  delete data_old;
  
  return(fSaved);
}


TreeItem* TreeView::getFirstMarkedItem()                          
{
  if (groupWithMarkedItems == (TreeGroupBase*)0)
    return((TreeItem*)0);
  else
  {
    TreeItem*      p = groupWithMarkedItems->itemFirstChild;
    
    while (p != (TreeItem*)0)
    {
      if (p->fMarked)
        return(p);
      p = p->itemNext;
    }
    return((TreeItem*)0);
  }
}


void TreeView::newViewBySearch(TreeGroupBase* group, 
                               SearchData     sd)                 
{
  // mark items
  group->markMatchingItems(sd);
  
  // link marked items to my root
  root->linkMarkedItems(group);
  
  // describe search
  {
    std::string tmp = "You have searched for all of the following words:\n";
    sd.AddWords(tmp);
    root->setComment(tmp.c_str());
  }
  
  // unmark items
  group->setMarkRec(false);
}


void TreeView::newViewWithoutText(TreeGroupBase* group)           
{
  // mark items
  group->markItemsWithoutText();
  
  // link marked items to my root
  root->linkMarkedItems(group);
  
  // unmark items
  group->setMarkRec(false);
}


void TreeView::newViewByDate(TreeGroupBase* group)                
{    
  // Alle TreePicBase unterhalb von group in eine Gruppe legen
  group->linkPicsInto(root);
    
  // zeitlich sortieren.
  root->sortChildrenChrono();
  
  // doppelte löschen
  root->deleteDuplicates();
  
  // Jetzt in Untergruppen aufteilen.
  root->subdivideChrono();
}


TreeView* TreeView::getSource()                                   
{
  return(source);
}


StringSet* TreeView::getHiddenKeywords()                          
{    
  return(&HiddenKeywords);
}


void TreeView::getMarkedItems(std::vector<TreeItem*>& list)       
{
  if (groupWithMarkedItems != (TreeGroupBase*)0)
  {
    TreeItem* p = groupWithMarkedItems->itemFirstChild;
  
    while (p != (TreeItem*)0)
    {
      if (p->fMarked)
        list.push_back(p);
      
      p = p->itemNext;
    }  
  }
}


void TreeView::exchangeThumbSettings(eThumbDir&   th,
                                     std::string& tds, 
                                     bool         fWrite) 
{
  if (source != (TreeView*)0)
  {
    source->exchangeThumbSettings(th, tds, fWrite);
  }
  else
  {
    if (fWrite)
    {
      thumbdir       = th;
      thumbdirselect = tds;
    }
    else
    {
      th  = thumbdir;
      tds = thumbdirselect;
    }
  }
}

void TreeView::markItems(TreeGroupBase* group, SearchData& sd)
{
  // bisherige Markierungen aufheben
  resetMarkItem((TreeItem*)0);
  // neu markieren
  if (group->markMatchingItems(sd, this))
    groupWithMarkedItems = group;
  else
    groupWithMarkedItems = 0;
}

void TreeView::RecursiveList(const std::vector<TreeItem*>& list, std::set<TreeItem*>& set)
{
  for (unsigned int n=0; n<list.size(); n++)
    list[n]->RecursiveList(set);
}
