// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "../../version.h"
#include "../config.h"

UserInterface* createNewWin()                             
{
  UserInterface* GUI;
  
  // Load file into a new window.
  char** a;
  a    = (char**)malloc(sizeof(char*)*1);
  char   name[] = TITLE_START;
  a[0] = (char*)(&name);
    
  GUI = new UserInterface();
  Fl_Window* window = GUI->make_window(1, a);        
  window->show(1, a);
  GUI->AfterCreation();
  
  return(GUI);
}


UserInterface* getUIByMenuBar(Fl_Widget* widget)          
{
  UserInterface* GUI = (UserInterface*)0;
  
  // Von welchem Fenster kommt die Anforderung?
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->menuBar == widget)
      GUI = UserInterface::windowList[n];
  
  return(GUI);
}


void cb_ende(Fl_Widget* widget, void* gui)                
{
  UserInterface* GUI = (UserInterface*)gui;
    
  if (GUI->askBeforeClosing() == 1)
  {
    GUI->saveOptions();
    GUI->window->hide();
    
    bool fVis = false;
    for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    {
      if (UserInterface::windowList[n]->window->visible())
        fVis = true;
    }
    if (!fVis && initHelpDlg != (Fl_Help_Dialog*)0)
      initHelpDlg->hide();
  }
}


void cb_new(Fl_Widget*, void* gui)                        
{
  createNewWin();
}


void cb_View(Fl_Widget* sender, void* action)             
{
  UserInterface* GUI = getUIByMenuBar(sender);
    
  //
  std::cout << "void cb_View(Fl_Widget* sender, void* action = " << (int)action << ")\n";
  
  switch ((int)action)
  {
   case UserInterface::SHOWPICS:
    {
      bool fNew = (GUI->myMainMenu[MENU_SHOWPICS].value() != 0);
      GUI->browser->setShowPics(fNew);
      
      GUI->selectCurrentItem();
    }
    break;
   case UserInterface::FULLSCREEN_PICS:
    GUI->setPicFullscreen();
    break;
      
   case UserInterface::FULLSCREEN:
    GUI->setFullscreen();
    break;
  }
}


void cb_picbox(Fl_Widget* sender, void* nAktion)          
{
  // which GUI does the sender belong to?
  UserInterface* GUI = (UserInterface*)0;
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->mainpane_pic == sender)
      GUI = UserInterface::windowList[n];

  if (GUI == (UserInterface*)0)
  {
    // todo: aufklären...
    //exit(-1);
    return;
  }
  
  TreeItem* currentPic = (TreeItem*)(((PicBox*)sender)->getPic());
  
//  enum PicBox::eCallback aktion = (PicBox::eCallback)nAktion;
  int aktion = (int)nAktion;
  
  switch (aktion)
  {
   case PicBox::PROPERTIES:
    GUI->setLastDisplayedAsLastPopupSingle();
    GUI->popupAction(PA_PROPERTIES);
    break;
   case PicBox::ADD_KEYWORDS:
    GUI->setLastDisplayedAsLastPopupSingle();
    GUI->popupAction(PA_ADD_KEYWORD);
    break;
   case PicBox::CONTEXT:
    std::cout << "callback: picbox context\n";
    GUI->showContextMenu(((PicBox*)sender)->nMX, ((PicBox*)sender)->nMY, currentPic);
    break;
   case PicBox::GROUP:
    GUI->displayItem(currentPic->itemParent);
    break;
   case PicBox::PICTURE_PREV:
    {
      TreeItem* item = currentPic->itemPrev;
      while (item != (TreeItem*)0 && (item->getType() != TreeItem::PIC || item->isHidden()))
        item = item->itemPrev;    
      if (item == (TreeItem*)0)    
        GUI->displayItem(currentPic->itemParent);    
      else
        GUI->displayItem(item);
    }
    break;
   case PicBox::PICTURE_NEXT:
    {
      TreeItem* item = currentPic->itemNext;
      while (item != (TreeItem*)0 && (item->getType() != TreeItem::PIC || item->isHidden()))
        item = item->itemNext;
      if (item == (TreeItem*)0)          
        GUI->displayItem(currentPic->itemParent);    
      else
        GUI->displayItem(item);
    }
    break;
   case PicBox::TOGGLE_MARK:
    {
      GUI->toggleMarkCurrentPic();
    }
    break;
  }
}


void cb_showItem(Fl_Widget* sender, void* gui)            
{
  UserInterface* GUI = (UserInterface*)gui;
  
  if (GUI->browser == sender)
  {
    GUI->displayItem(GUI->browser->itemSelected, false);
  }
  else
  {
    // Es war ein Callback aus der Gruppenansicht.
    // Welcher Button war es? Man kann den Zeiger auf den Tooltip 
    // zur Identifikation benutzen!
    int nSize = GUI->groupviewdata.size();
    for (int n=0; n<nSize; n++)
    {
      if (sender->tooltip() == GUI->groupviewdata[n].tooltip)
        GUI->displayItem(GUI->groupviewdata[n].pic);        
    }
  }
}


void cb_textbox(Fl_Widget* sender, void* gui)             
{
  UserInterface* GUI = (UserInterface*)gui;
  TextBox*       tb  = (TextBox*)sender;
  
  GUI->followLink(tb->getLinkText());
}


void loadfile(UserInterface* GUI,
              const char*    filename)                    
{
  if (GUI->hasData())
  {
    GUI = createNewWin();
  }      
  
  GUI->loadFile(filename);  
}


void cb_loadfile(Fl_Widget*, void* gui)                   
{
  UserInterface* GUI = (UserInterface*)gui;
  
  char*        newfile;
  std::string  pathToStartFrom;
    
  pathToStartFrom = GUI->myFilename;
  while (pathToStartFrom.length() > 0 && pathToStartFrom[pathToStartFrom.length()-1] != '/')
    pathToStartFrom.replace(pathToStartFrom.length()-1, 1, "");
  
  newfile = fl_file_chooser("Open file", "*.pg.xml", pathToStartFrom.c_str());
    
  if (newfile != (char*)0)
  {
    loadfile(GUI, newfile);
  }
}


void cb_loadrecentfile(Fl_Widget* widget, void* userdata) 
{
  UserInterface* GUI = getUIByMenuBar(widget);
  
  loadfile(GUI, GUI->opts->recentFile((int)userdata).c_str());  
}


void cb_popup(Fl_Widget* widget, void* userdata)          
{
  UserInterface* GUI = (UserInterface*)0;

  Fl_Tooltip::enable(1);
  
  // Von welchem Fenster kommt die Anforderung?
  for (unsigned int n=0; n<UserInterface::windowList.size(); n++)
    if (UserInterface::windowList[n]->popup == widget)
      GUI = UserInterface::windowList[n];
  
  GUI->popupAction((int)userdata);
}


void cb_savefile(Fl_Widget*, void* gui)                   
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->save(false);
}


void cb_savefileas(Fl_Widget*, void* gui)                 
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->save(true);
}


void cb_options(Fl_Widget*, void* gui)                    
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->optionsdlg();
}


void cb_albset(Fl_Widget*, void* gui)                    
{
  UserInterface* GUI = (UserInterface*)gui;

  GUI->albumsettingsdlg();
}


void cb_Help(Fl_Widget*, void* value)                     
{
  switch ((int)value)
  {
   case 1:
    {
      fl_message("Version Number: " FOTOALBUMVERSION
                 "\nJens W. Wulf 2005-2011"
                 "\njens_w2 at gmx dot net\n\n"
                 "install prefix: "                                   STR_INSTALL_PREFIX "\n"
                 "build type: "                                       STR_CMAKE_BUILD_TYPE "\n"
                 "libgraphicsmagick (thumbnail creation, rotation): " STR_USE_LIBGRAPHICSMAGICK "\n"
                 "integrated jpegtran (rotation): "                   STR_USE_INTEGRATED_JPEGTRAN "\n"
                 "libexif (EXIF data): "                              STR_USE_LIBEXIF "\n"
                 "libexiv2 (EXIF and IPTC data): "                    STR_USE_EXIV2 "\n"
                 "pthreads: "                                         STR_USE_PTHREADS "\n"
                 );
    }
    break;
   default:
    {
      bool fDone = false;
      std::string strPath = GUIMisc::getHelpPath("", "fa_index.htm");

      if (WIN32_VAL != 0 && !fDone)
      {
        if (system(strPath.c_str()) == 0)
          fDone = true;
      }
      
      if (WIN32_VAL == 0 && !fDone)
      {
        if (system(("x-www-browser " + strPath).c_str()) == 0)
          fDone = true;
      }
      
      if (WIN32_VAL == 0 && !fDone)
      {
        if (system(("xdg-open " + strPath).c_str()) == 0)
          fDone = true;
      }
      
      if (!fDone)
      {
        Fl_Help_Dialog* dlg = new Fl_Help_Dialog();            
        dlg->load(strPath.c_str());
        dlg->show();
      }
    }
    break;
  }
}

void cb_idle(void* foo)
{
  UserInterface* gui = (UserInterface*)(foo);  
  gui->Idle();
}
