// -*- mode: c; mode: fold -*-
// 
/*
 * Copyright (C) 2005, 2008-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "itempropdlg.h"

#include <iostream>
#include <string.h>
#include <FL/fl_ask.H>

#include "../base/treepicbase.h"
#include "../shared/lib_conversions.h"
#include "gui.h"
#include "dateselect.h"
#include "addkwdlg.h"
#include "../base/basetools.h"

void kill_selection(Fl_Text_Editor* e) 
{
  if (e->buffer()->selected()) 
  {
    e->insert_position(e->buffer()->primary_selection()->start());
    e->buffer()->remove_selection();
  }
}

int kf_backspace_word(int, Fl_Text_Editor* e)
{
  if (!e->buffer()->selected())
  {
    Fl_Text_Buffer* buf = e->buffer();
    
    int nMoves = 0;
    bool fGo = true;
    int nStep = 1;
    int pos = e->insert_position() - 1;
    if (pos < 0)
      pos = 0;
    // delete until some word-character is left of cursor
    while (fGo && nStep)
    {
      if (isalpha(buf->character(pos)) == 0)
      {
        nStep = e->move_left();
        nMoves += nStep;
      }
      else
        fGo = false;
      pos = e->insert_position() - 1;
      if (pos < 0)
        pos = 0;
    }
    // delete until some none-word-character is left of cursor
    fGo = true;
    nStep = 1;
    while (fGo && nStep)
    {
      if (isalpha(buf->character(pos)) != 0)
      {
        nStep = e->move_left();
        nMoves += nStep;
      }
      else
        fGo = false;
      pos = e->insert_position() - 1;
      if (pos < 0)
        pos = 0;
    }
     
    if (nMoves)
      e->buffer()->select(e->insert_position(), e->insert_position()+nMoves);
  }
  
  kill_selection(e);
  
  e->show_insert_position();
  e->set_changed();
  if (e->when()&FL_WHEN_CHANGED)
    e->do_callback();
  return 1;
}

void cb_edit(Fl_Widget* widget, void* userdata)
{
  Fl_Text_Editor* e = (Fl_Text_Editor*)(userdata);
  Fl_Text_Buffer* b = e->buffer();
  
  int pos = e->insert_position() - 1;
  int linepos = 0;
  int nIndent = 0;
  {
    int pos_table_start = 0;
    int pos_notspace = 0;
    while (pos > 0 && b->character(pos) != '\n')
    {
      if (b->character(pos) == '|')
        pos_table_start = pos;
      if (isspace(b->character(pos)) == 0)
        pos_notspace = pos;
      linepos++;
      pos--;
    }
    
    if (pos+1 == pos_table_start)
      linepos = 0; // don't break table lines
    nIndent = pos_notspace - pos - 1;
  }
  
  if (linepos > 80)
  {
    pos = e->insert_position() - 1;
    while (pos > 0 && b->character(pos) != '\n' && b->character(pos) != ' ')
      pos--;
    if (b->character(pos) == ' ')
      b->replace(pos, pos+1, "\n");
    for (; nIndent>0; nIndent--)
      b->insert(pos+1, " ");         
  }
}

void cb_close(Fl_Widget* widget, void* userdata)
{
  // force the user to click "cancel" or "apply"
}

void cb_ipd_cancel(Fl_Widget* widget, void* userdata)
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  
  std::cout << "ItemPropertyDlg: cancel (" << dlg << ")\n";
  
  dlg->window->hide();
  delete dlg;
}

void cb_ipd_addkw(Fl_Widget* widget, void* userdata) 
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  dlg->AddKw();
}

void cb_ipd_ok(Fl_Widget* widget, void* userdata)     
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  std::cout << "ItemPropertyDlg: ok (" << dlg << ")\n";
  if (dlg->saveValues() == 0)
  {
    dlg->checkErrors();
    dlg->window->hide();
    delete dlg;
  }
}

void cb_ipd_id(Fl_Widget* widget, void* userdata)     
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  std::cout << "ItemPropertyDlg: id (" << dlg << ")\n";
  dlg->createID();
}

void cb_ipd_date(Fl_Widget* widget, void* userdata)   
{
  ItemPropertyDlg* dlg = (ItemPropertyDlg*)userdata;
  std::cout << "ItemPropertyDlg: date (" << dlg << ")\n";
  
  int y, m, d;
  
  // das aktuelle Datum übergeben, falls es eins gibt:
  TreePicBase* pic = ((TreePicBase*)(dlg->treeitem));
  
  if (pic->hasDate())
  {
    JWWDateTime dt;
    
    dt.setDate(dlg->inp_date->value());
    
    d = dt.getDay();
    m = dt.getMonth();
    y = dt.getYear();    
  }
  else    
    d = -1;
  
  DateSelect* dateselect = new DateSelect(&y, &m, &d);
  
  dateselect->show();
  while (dateselect->visible())
    Fl::wait();
  
  if (d != -1)
  {    
    dlg->inp_date->value(JWWDateTime::getSomeDateAsString(JWWDateTime::assembleDate(y, m, d)).c_str());
  }
  delete dateselect;
}

ItemPropertyDlg::ItemPropertyDlg(TreeItem* myItem, 
                                 TreeView* myTreeView) 
{
  std::cout << "ItemPropertyDlg: constructor (" << this << ")\n";
  treeitem = myItem;
  treeview = myTreeView;
}


ItemPropertyDlg::~ItemPropertyDlg()              
{
  std::cout << "ItemPropertyDlg: destructor (" << this << ")\n";
  
  inp_keywords->buffer(0);
  delete buf_keywords;
  
  inp_comment->buffer(0);
  delete buf_comment;
  
  Fl::delete_widget(window);
}

void ItemPropertyDlg::AddKw()
{
  std::set<std::string> kwset;        
  treeview->root->kw_setGroup(kwset);
  AddKwDlg* dlg = new AddKwDlg(kwset);
  if (dlg->showDlg())
  {
    const char* kwstringptr = dlg->ti_words->value();
    if (kwstringptr != (const char*)0)
    {
      std::set<std::string> newkwlist = BaseTools::KeywordList_Set(kwstringptr);
      
      if (!newkwlist.empty())
      {
        std::set<std::string>           oldkwlist = BaseTools::KeywordList_Set(buf_keywords->text());
        std::set<std::string>::iterator it;  
        
        oldkwlist.insert(newkwlist.begin(), newkwlist.end());
        buf_keywords->remove(0, buf_keywords->length()); // clear buffer
        for (it = oldkwlist.begin(); it != oldkwlist.end(); it++)
        {
          buf_keywords->append((*it).c_str());
          buf_keywords->append(",\n");
        }                       
      }
    }
  }

  delete(dlg);
}

void ItemPropertyDlg::make_window()              
{
  int nY  = 0;
  int nTH = (UserInterface::opts->nFontSize*3)>>1;
  int nCH;
  const int nW = 750;
  
  unsigned int inside = 0;
  unsigned int insideandbelow = 0;
  
  {
    int nRows;
    
    if (treeitem->getType() == TreeItem::PIC)
    {
      nRows = 15;
    }
    else
    {
      nRows = 14;
      ((TreeGroupBase*)treeitem)->countNonGroupItems(inside, insideandbelow);
    }
    
    if (treeitem->getLinkType() != TreeItem::DATA)
      nRows++;
    
    nCH = 550 - nRows * nTH;
  }
  
  window = new Fl_Double_Window(750, 550);
  window->label("Properties");
  
  if (treeitem->getLinkType() != TreeItem::DATA)
  {
    std::string info = "This item is a link. It points to ";
    info += itoStr(treeitem->getLinkID(), ' ', 1);
    out_linkinfo = new Fl_Output(0, nY, nW, nTH);
    out_linkinfo->value(info.c_str());
    nY += nTH;
  }
  
  //
  inp_title    = new Fl_Input(0, nY+nTH, nW, nTH);
  inp_title->value(treeitem->getText().c_str());
  if (treeitem->getType() == TreeItem::PIC)
  {
    inp_title->label("Path:");
    inp_title->readonly(1);
  }
  else
  {
    namelabel = "Name:";
    
    if (treeitem->getType() != TreeItem::PIC)
    {
      namelabel += "         (";
      namelabel += itoStr(inside, ' ', 1);
      namelabel += "/";
      namelabel += itoStr(insideandbelow, ' ', 1);
      namelabel += ")";
    }
    
    inp_title->label(namelabel.c_str());
  }
  inp_title->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  nY += 2*nTH;
  
  // 
  buf_comment  = new Fl_Text_Buffer();
  buf_comment->append(treeitem->getComment().c_str());
  inp_comment  = new Fl_Text_Editor(0, nY+nTH, nW, nCH);
  inp_comment->label("Comment:");
  inp_comment->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  inp_comment->buffer(buf_comment);  
  nY += nTH + nCH;    
  
  // 
  buf_keywords = new Fl_Text_Buffer();
  {
    std::vector<std::string> kwlist;
    treeitem->kw_list(kwlist);
    for (unsigned int n=0; n<kwlist.size(); n++)
      buf_keywords->append((kwlist[n] + ",\n").c_str());
  }
  inp_keywords = new Fl_Text_Editor(0, nY+nTH, nW-100, 6*nTH);
  inp_keywords->label("Keywords:");
  inp_keywords->buffer(buf_keywords);
  inp_keywords->align(FL_ALIGN_LEFT | FL_ALIGN_TOP);
  btn_kwadd    = new Fl_Button(nW-100, nY+nTH, 100, nTH, "Add...");
  btn_kwadd->callback( (Fl_Callback*)cb_ipd_addkw, (void*)this );
  nY += (15*nTH)>>1;
  
  if (treeitem->getType() == TreeItem::PIC)
  {
    int nDTW = nW / 5;
    
    btn_date    = new Fl_Button(0, nY, nDTW, nTH, "Date:");
    btn_date->callback( (Fl_Callback*)cb_ipd_date, (void*)this );
    
    //
    inp_date    = new Fl_Input(nDTW, nY, nDTW, nTH);
    inp_date->value(((TreePicBase*)treeitem)->getDateString().c_str());
    
    //
    inp_time    = new Fl_Input(nW>>1, nY, nDTW, nTH);
    inp_time->value(((TreePicBase*)treeitem)->getTimeString().c_str());
    inp_time->label("Time:");
    inp_time->align(FL_ALIGN_LEFT);
    nY += (3*nTH)>>1;
  }
  
  //
  uID    = treeitem->getID();
  btn_ID = new Fl_Button(nW>>2, nY, nW>>1, nTH, "ID");
  if (uID != 0)
  {
    strcpy(IDString, "ID: ");
    strcat(IDString, itoStr(uID, ' ', 1).c_str());
    btn_ID->label(IDString);
  }
  else
  {
    btn_ID->label("Create ID");
    btn_ID->callback( (Fl_Callback*)cb_ipd_id, (void*)this );
  }
  nY += nTH;
  
  window->end();
  window->resizable(inp_comment);
  
  //
  nY += nTH;
  btn_Cancel    = new Fl_Button(0, nY, 100, nTH, "Cancel");
  btn_Cancel->callback( (Fl_Callback*)cb_ipd_cancel, (void*)this );
  
  //
  btn_OK        = new Fl_Button(100, nY, 100, nTH, "&Apply");
  btn_OK->callback( (Fl_Callback*)cb_ipd_ok, (void*)this );
  
  window->add(btn_Cancel);
  window->add(btn_OK);
  
  window->resizable(inp_comment);
  
  window->set_modal();
  window->show();
  window->callback((Fl_Callback*)cb_close, (void*)this );
  
  if (treeitem->getType() == TreeItem::PIC)
    Fl::focus(inp_comment);
  else
    Fl::focus(inp_title);
  
  checkErrors();
  
  {
    inp_comment->remove_key_binding(FL_BackSpace, FL_TEXT_EDITOR_ANY_STATE);
    inp_comment->add_key_binding(FL_BackSpace, 0, Fl_Text_Editor::kf_backspace);
    inp_comment->add_key_binding(FL_BackSpace, FL_CTRL, kf_backspace_word);
    
    inp_comment->callback( (Fl_Callback*)cb_edit, (void*)(inp_comment));
    inp_comment->when(FL_WHEN_CHANGED);
  }
}

int  ItemPropertyDlg::saveValues()               
{
  const char* text;
  int         nRetVal = 0;
  
  
  if (treeitem->getType() == TreeItem::GROUP)
    ((TreeGroupBase*)treeitem)->setTitle(inp_title->value());
  
  {
    treeview->showLinkableItems();
    
    LinkInfo*  li       = treeview->getLinkInfoList();
    int        nRetCode = treeitem->countCommentLinks(li, false);
    
    if (nRetCode & eLiErrUnderflow)
      fl_alert("critical internal error: link count underflow");
    
    text  = buf_comment->text();
    treeitem->setComment(text);
    //  free(text); // see docs! should be done!
    treeitem->countCommentLinks(li, true);
    
    treeview->showLinkableItems();    
  }

  {    
    // clean up keywords
    std::string                     stringbuf = "";
    std::set<std::string>           kwlist    = BaseTools::KeywordList_Set(buf_keywords->text());
    std::set<std::string>::iterator it;  
        
    for (it = kwlist.begin(); it != kwlist.end(); it++)
    {
      stringbuf.append(*it);
      stringbuf.append(std::string(","));
    }                       
    treeitem->setKeywords(stringbuf.c_str());    
  }
  //  free(text); // see docs! should be done!
    
  if (treeitem->getType() == TreeItem::PIC)
  {
    if (((TreePicBase*)treeitem)->setDate(inp_date->value()))
    {
      fl_alert("Error in date format");
      nRetVal = -1;
    }
    if (((TreePicBase*)treeitem)->setTime(inp_time->value()))
    {
      fl_alert("Error in time format");
      nRetVal = -1;
    }
  }
  
  if (treeitem->getID() != uID)
  {
    treeitem->setID(uID);
    treeview->newLinkableItem(treeitem);
  }
  
  return(nRetVal);
}

void ItemPropertyDlg::createID()                 
{
  uID = treeview->getIDMax() + 1;
  strcpy(IDString, "ID: ");
  strcat(IDString, itoStr(uID, ' ', 1).c_str());
  btn_ID->label(IDString);
}

void ItemPropertyDlg::checkErrors()              
{
  std::vector<unsigned int> ids;
  std::vector<unsigned int> idserr;
  LinkInfo*                 li = treeview->getLinkInfoList();
  
  treeitem->appendCommentIDs(ids);
  
  for (unsigned int n=0; n<ids.size(); n++)
  {
    if (li->getItemWithID(ids[n]) == (TreeItem*)0)
      idserr.push_back(ids[n]);
  }
  
  if (idserr.size())
  {
    std::string msg = "Link IDs to which no item exists:\n";
    for (unsigned int n=0; n<idserr.size(); n++)
    {
      msg += itoStr(idserr[n], ' ', 1) + "\n";
    }
    fl_alert(msg.c_str());
  }    
}

