/*
 * Copyright (C) 2005, 2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "picbutton.h"

#include <iostream>
#include <FL/Fl.H>
#include <FL/fl_draw.H>

#include "picgroup.h"
#include "itemchange.h"
#include "../base/treeview.h"

PicButton::PicButton(int X,int Y,int W,int H,const char *l)
  : Fl_Button(X,Y,W,H,l)
{
}

PicButton::~PicButton()
{
//  std::cout << "PicButton::~PicButton()\n";
}

void PicButton::draw()
{
  if (myPic->fMarked)
    fl_draw_box(FL_FLAT_BOX,   x(), y(), w(), h(), FL_SELECTION_COLOR);
  else    
    fl_draw_box(FL_BORDER_BOX, x(), y(), w(), h(), FL_BACKGROUND2_COLOR);
  draw_label();
}

int PicButton::handle(int event)
{
  int X,Y,W,H; 
  
  X = x();
  Y = y();
  W = w();
  H = h();
  
  if (!Fl::event_inside(X,Y,W,H))
    return 0;
  
  switch (event) 
  {
   case FL_ENTER:
    return 1;
    break;
    
   case FL_PUSH:
    return 1;
    break;
    
   case FL_RELEASE:
    if (Fl::event_button() == FL_RIGHT_MOUSE)
    {
      // context menu
      ((ItemChange*)parent())->showContextMenu(Fl::event_x_root(), Fl::event_y_root(), myPic);
    }
    else
    {
      if ((Fl::event_state() & FL_CTRL) != 0)
      {
        if (myPic != (TreeItem*)0)
        {
//          std::cout << "toggle marked\n";
          ((PicGroup*)parent())->getData()->toggleMarkItem(myPic);
          ((PicGroup*)parent())->sayItemChanged(myPic);
          if (myPic->fMarked)
            ((PicGroup*)parent())->lastMarkedItemCtrl = (void*)myPic;
          redraw();
        }
      }
      else if ((Fl::event_state() & FL_SHIFT) != 0)
      {
        if (myPic != (TreeItem*)0)
        {
//          std::cout << "toggle marked range\n";
          ((PicGroup*)parent())->setMarkItemUpTo(myPic);
          ((PicGroup*)parent())->sayItemChanged(myPic);
          redraw();
        }
      }
      else
      {
        do_callback();
      }
    }
//    std::cout << "PicButton: end of handle\n";
    return 1;
    break;
  }
  
  return Fl_Button::handle(event);
}

void PicButton::item(TreePicBase* p)
{
  myPic = p;
}

void PicButton::redrawIf(TreeItem* ptr)
{
  if (ptr == myPic)
    redraw();
}

void PicButton::setLabel(std::string somestring)
{
  mylabel = somestring;
  label(mylabel.c_str());
}
