// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

/**
 * Möglicherweise ist da ein Bug im Fl_Scroll oder dem dabei benutzten Scrollbar,
 * jedenfalls bekomme ich eine fehlerhafte Anzeige bei vielen Bildern in einer 
 * Gruppe und großer Vorschau:
 * 
 * 475 Bilder in einer Gruppe
 * 
 * ok mit 50:
 * 
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize    = 56
     PicGroup::resize:  nPicsPerRow = 14
     PicGroup::resize:  nTBH        = 90
 * 
 * ok mit 100:
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize    = 106
     PicGroup::resize:  nPicsPerRow = 7
     PicGroup::resize:  nTBH        = 90
 * 
 * ok mit 150:
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize    = 156
     PicGroup::resize:  nPicsPerRow = 5
     PicGroup::resize:  nTBH        = 90
 * 
 * ok mit 200: -> 24462 Pixel Gesamthöhe
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize    = 206
     PicGroup::resize:  nPicsPerRow = 4
     PicGroup::resize:  nTBH        = 90
 * 
 * fehlerhaft mit 250: -> 40533 Pixel Gesamthöhe
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize    = 256
     PicGroup::resize:  nPicsPerRow = 3
     PicGroup::resize:  nTBH        = 90
 * 
 * 
 * 346 Bilder in einer Gruppe
 * 
 * ok mit 250: -> 29525 Pixel Gesamthöhe
 * void PicGroup::resize(296, 28, 902, 1537)
     PicGroup::resize:  nPixelSize  = 256
     PicGroup::resize:  nPicsPerRow = 3
     PicGroup::resize:  nTBH        = 90

 */


#include "picgroup.h"

#include <iostream>
# include <FL/Fl.H>


#include "callback.h"
#include "picbutton.h"
#include "../base/treeitem.h"
#include "../base/treeview.h"

// todo
#define SCROLLBARWIDTH 20

PicGroup::PicGroup(int X,int Y,int W,int H,const char *l)
  : Fl_Scroll(X,Y,W,H,l)
{
  textbox  = (TextBox*)0;
  type(Fl_Scroll::VERTICAL_ALWAYS);
}

PicGroup::~PicGroup()
{
}

void PicGroup::resize(int x, int y, int w, int h) 
{
  std::cout << "void PicGroup::resize(" << x << ", " << y << ", " << w << ", " << h << ")\n";
  
  int nTBH = 0;
  
  if (textbox != (TextBox*)0)
  {
    textbox->recalcLayoutInto(x, y, w-SCROLLBARWIDTH, h);
    nTBH = textbox->h();
  }
  

  int        nSize         = children();  
  int        nPixelSize    = 0;
  Fl_Widget* widget;
  
  for (int n=0; n<nSize && nPixelSize == 0; n++)
  {
    widget = child(n);
    if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      nPixelSize = widget->w();
  }
  
  std::cout << "     PicGroup::resize:  nPixelSize  = " << nPixelSize << "\n";

  if (nSize > 0 && nPixelSize > 0)
  {    
    int nGap            = 5;
    int nScrollbarWidth = SCROLLBARWIDTH; // todo
    int nPicsPerRow     = (w - nScrollbarWidth)/ (nPixelSize+nGap);
    int m  = 0;
    int ly = nGap + y + nTBH;
      
    std::cout << "     PicGroup::resize:  nPicsPerRow = " << nPicsPerRow << "\n";
    std::cout << "     PicGroup::resize:  nTBH        = " << nTBH << "\n";
    
    for (int n=0; n<nSize; n++)
    {
      widget = child(n);
      
      if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      {
        widget->resize(x + nGap + m * (nPixelSize+nGap), ly, nPixelSize, nPixelSize);
        m++;
        if (m == nPicsPerRow)
        {
          m = 0;
          ly += nPixelSize + nGap;
        }
      }
    }
  }
    
  Fl_Scroll::resize(x, y, w, h);
}


void PicGroup::draw()                             
{
  Fl_Scroll::draw();
//  textbox->draw();
}


void PicGroup::setDescr(std::string newdescr)     
{
  if (textbox == (TextBox*)0)
  {
    textbox = new TextBox(x(), y(), w()-SCROLLBARWIDTH, h());
    textbox->labelsize(labelsize());
    // set up callback
    textbox->callback( (Fl_Callback*)cb_textbox, user_data());
  }

  textbox->setDescr(newdescr.c_str());
  textbox->recalcLayoutInto(x(), y(), w()-SCROLLBARWIDTH, h());
  insert(*textbox, 0);
}


void PicGroup::clear()                            
{
  Fl_Scroll::clear();
  textbox             = (TextBox*)0;
  lastMarkedItemCtrl  = (void*)0;
  lastMarkedItemShift = (void*)0;
}


void PicGroup::itemChanged(TreeItem* ptr)         
{
  // std::cout << "an item has changed state\n";
  
  int        nSize         = children();  
  Fl_Widget* widget;
  
  for (int n=0; n<nSize; n++)
  {
    widget = child(n);
    if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
    {
      // Maybe all of them have been unmarked. So the following line 
      // is not sufficient:
      // ((PicButton*)widget)->redrawIf(ptr);
      ((PicButton*)widget)->redraw();
    }
  }
  
}


int  PicGroup::setMarkItemUpTo(void* ptr)         
{
  if (lastMarkedItemCtrl == (void*)0)
    return(-1);  
  
  int        nSize    = children();  
  int        nOK      = 0;
  bool       fSet     = false;
  bool       fReset   = false;
  Fl_Widget* widget;
  
  // Can I find both items?
  for (int n=0; n<nSize; n++)
  {
    widget = child(n);
    
    if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
    {
      if (((PicButton*)widget)->myPic == ptr)
        nOK++;
      else if (((PicButton*)widget)->myPic == lastMarkedItemCtrl)
        nOK++;
    }
  }
  
  if (nOK == 2)
  {
    TreeItem* i;
    bool      fShiftUsed;
    bool      fCtrlUsed;
    
    nOK = 0;
      
    // Unmark and mark
    for (int n=0; n<nSize && nOK < 2; n++)
    {
      widget = child(n);
      if (widget != &scrollbar && widget != &hscrollbar && widget != textbox)
      {
        i = ((PicButton*)widget)->myPic;
        
        fShiftUsed = false;
        fCtrlUsed  = false;

        if ((i == ptr || i == lastMarkedItemShift) && fReset == false)
        {
          fReset     = true;
          fShiftUsed = true;
        }                        
        
        if ((i == ptr || i == lastMarkedItemCtrl) && fSet == false)
        {
          fSet      = true;
          fCtrlUsed = true;
        }
                
        if (fReset && !fSet)
        {
          treeview->resetMarkItem(i);
          widget->redraw();
        }
        else if (fSet)
        {
          treeview->setMarkItem(i);
          widget->redraw();
        }
        
        if ((i == ptr || i == lastMarkedItemCtrl) && fSet && !fCtrlUsed)
        {
          fSet = false;
          nOK++;
        }
        if ((i == ptr || i == lastMarkedItemShift) && fReset && !fShiftUsed)
        {
          fReset = false;
          nOK++;
        }        
      }
    }
    lastMarkedItemShift = ptr;
    return(0);
  }
  return(-1);
}

