// -*- mode: c; mode: fold -*-

/*
 * Copyright (C) 2005,2008  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include "pictreebrowser.h"


#include "../base/treeitem.h"
#include "gui.h"

#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include <iostream>


PicTreeBrowser::PicTreeBrowser(int X,int Y,int W,int H,const char*l)
: Fl_Browser_(X,Y,W,H,l) 
{  
  type(FL_HOLD_BROWSER);
  
  fShowPics    = false;
  itemSelected = (TreeItem*)0;
}

void* PicTreeBrowser::item_first() const                    
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->root));
  else
    return((void*)0);
}


void* PicTreeBrowser::item_next(void *l) const              
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->browserItemNext((TreeItem*)l, fShowPics)));
  else
    return((void*)0);
}


void* PicTreeBrowser::item_prev(void *l) const              
{
  if (treeview != (TreeView*)0)
    return((void*)(treeview->browserItemPrev((TreeItem*)l, fShowPics)));
  else
    return((void*)0);
}


int   PicTreeBrowser::item_height(void *l) const            
{
  return(nItemHeight);
}


void  PicTreeBrowser::textsize(uchar s)                     
{
  Fl_Browser_::textsize(s);
  
  // calculate things  
  nItemHeight = (10*s) >> 3;
  
  // Das Quadrat sollte eine ungerade Höhe haben,
  // damit das Kreuz darin wirklich in der Mitte ist.
  nSizeBox = s;
  if ((nSizeBox & 1) == 0)
    nSizeBox--;
  
  nLengthDash = (s * 3) >> 3;  
  if ((nLengthDash & 1) == 0)
    nLengthDash--;
  nD          = (( (10*s) >> 3) - nSizeBox) >> 1;
  nTextsize   = s;
  nGroupAdd   = (s * 3) >> 1;
  
  std::cout << "PicTreeBrowser layout:\n";
  std::cout << "  nTextsize   = " << nTextsize << "\n";
  std::cout << "  nItemHeight = " << nItemHeight << "\n";
  std::cout << "  nSizeBox    = " << nSizeBox << "\n";
  std::cout << "  nLengthDash = " << nLengthDash << "\n";
  std::cout << "  nD          = " << nD << "\n";
  std::cout << "  nGroupAdd   = " << nGroupAdd << "\n";
}


int   PicTreeBrowser::handle(int e)                         
{
  TreeItem *l;
  
  // get box INSIDE the scrollbars
  int X,Y,W,H;  
  bbox(X,Y,W,H);
  
  switch (e) 
  {
   case FL_PUSH:
    // to make the Fl_Browser_ handle the scrollbars:
    if (
        !Fl::event_inside(X,Y,W,H)
        )
      break;
    
    return 1;
    break;
    
   // only to make sure Fl_Browser_ will not handle it
   case FL_DRAG:    
    return(1);    
    
   case FL_RELEASE:
    l = (TreeItem*)find_item(Fl::event_y());
    if (l != (TreeItem*)0)
    {
      int  nXRelease = Fl::event_x() + hposition() - leftedge();
      bool fRedraw   = false;
      bool fContext  = false;
      int  nXBoxMin  = 3 + l->treeDepth() * 2 * nTextsize;
      int  nXBoxMax  = nXBoxMin + nTextsize;
      
      if (Fl::event_button() == FL_RIGHT_MOUSE)
      {
        fContext = true;
      }
      else
      {
        if (nXRelease < nTextsize && l->itemParent != (TreeItem*)0)
        {
          treeview->toggleMarkItem(l);
          fRedraw = true;
          // the PicButton needs to be redrawn, too:
          sayItemChanged(l);
        }
        
        if (l->getType() == TreeItem::GROUP &&
            l->itemParent != (TreeItem*)0 &&
            nXRelease >= nXBoxMin &&
            ((TreeGroupBase*)l)->canExpand(fShowPics))
        {
          TreeGroupBase* g = (TreeGroupBase*)l;
          if (nXRelease >= nXBoxMax)
          {
            g->fExpanded = true;
          }
          else
          {
            g->fExpanded = !g->fExpanded;
          }
          fRedraw = true;
        }        
      }            

      if (nXRelease >= nXBoxMax)
      {
        // select
        // Mark item as selected
        itemSelected = l;
        // 
        do_callback();
        fRedraw = true;
      }
        
      if (fRedraw)
        redraw();
      
      if (fContext)
        gui->showContextMenu(Fl::event_x_root(), Fl::event_y_root(), l);
        
      return(1);        
    }
  }
  return Fl_Browser_::handle(e);
}


int   PicTreeBrowser::item_width(void *v) const             
{
  TreeItem* item     = (TreeItem*)v;
  int       X        = 3 + item->treeDepth() * 2 * nTextsize;

  if (item->getType() == TreeItem::GROUP)
    X += nGroupAdd;
    
  return( (int)(fl_width( item->getText().c_str() ))         + X);
}


void  PicTreeBrowser::item_draw(void *v, 
                                int X, int Y, 
                                int, int) const             
{
  TreeItem* item     = (TreeItem*)v;
  Fl_Color lcol = textcolor();
  if (item_selected(v))
    lcol = fl_contrast(lcol, selection_color());
  fl_color(lcol);
  
  
  X += 3;
  
  if (item->fMarked)
  {
    fl_line_style(0, 2);
    fl_line(X,            Y+nD, X+nSizeBox-1, Y+nD+nSizeBox-1);
    fl_line(X+nSizeBox-1, Y+nD, X,            Y+nD+nSizeBox-1);
    fl_line_style(0);
  }
  else
  {
    fl_line_style(0);
    fl_rect(X+1, Y+nD+1, nSizeBox-2, nSizeBox-2);
  }
  
  X += item->treeDepth() * 2 * nTextsize;
  
  if (item->getType() == TreeItem::GROUP)
  {
    TreeGroupBase* g        = (TreeGroupBase*)item;        
  
    int  nMitte   = (nSizeBox>>1);
    int  nAbstand = (nSizeBox - nLengthDash)>>1;        
    
    fl_rect(X, Y+nD, nSizeBox, nSizeBox);
    
    if (g->fExpanded)
      fl_line(X+nAbstand, Y+nD+nMitte,   X+nAbstand+nLengthDash-1, Y+nD+nMitte);
    else if (g->canExpand(fShowPics))
    {
      fl_xyline(X+nAbstand, Y+nD+nMitte,   X+nAbstand+nLengthDash-1);
      fl_yxline(X+nMitte,   Y+nD+nAbstand, Y+nD+nAbstand+nLengthDash-1);
    }
    X += (nTextsize * 3) >> 1;
  }
  
  if (item->getLinkType() == TreeItem::DATA)    
    fl_font(textfont(), nTextsize);
  else
    fl_font(textfont() | FL_ITALIC, nTextsize);
  fl_draw(item->getText().c_str(),
          X, Y + nTextsize - ((int)(fl_descent())>>2));
}


void  PicTreeBrowser::setShowPics(bool fNew)                
{
  std::cout << "void  PicTreeBrowser::setShowPics(" << fNew << ")\n";  
  fShowPics = fNew;
  
  if (!fShowPics)
    treeview->root->collapsePicsHidden();
  
  redraw();
}


void  PicTreeBrowser::itemChanged(TreeItem* ptr)            
{
  std::cout << "void  PicTreeBrowser::itemChanged(TreeItem* ptr): an item has changed state\n";
  std::cout.flush();
  // it is not enough to redraw this single line -- other might have been reset
  // redraw_line(ptr);
  redraw();
}


