/*
 * Copyright (C) 2005, 2008-2009  Jens Wilhelm Wulf, j dot w dot wulf at gmx dot net
 * Licensed under the GNU General Public License version 2.
 */

#include <stdio.h>
#include <cstring>
#include "searchdlg.h"
#include "texttreeitem.h"
#include "misc.h"

void cb_browser(KWSRCHBROWSER* sender, SearchDlg* searchdlg)
{
  printf("search dialog is %p\n", searchdlg);
  printf("browser is       %p\n", sender);
  
#if (BROWSER == 0)
  int item = sender->value();  
  if (item)
  {
    const char* content = (const char*)(sender->text(item));
    printf("Selected line %i\n", item);
    if (content)
      searchdlg->AddKw(content);
  }
#else
  searchdlg->AddKw(sender->selected_path.c_str());
#endif
}

void SearchDlg::AddKw(const char* item)
{
  std::string buf = ti_words->value();
  if (strlen(item))
  {
    buf += ",";
    buf += item;
    ti_words->value(buf.c_str());
  }
}

SearchDlg::SearchDlg(std::set<std::string>& kwset)
{
  TextTreeItem* root = TextTreeItem::CreateFromSet(kwset);
  
  printf("Baum erzeugt\n");
  
  Fl_Double_Window* w;
  { Fl_Double_Window* o = myWindow = new Fl_Double_Window(785, 550, "Search");
    w = o;
    o->user_data((void*)(this));    
    { Fl_Input* o = ti_words = new Fl_Input(15, 30, 755, 25, "Words to search for (separated by \',\'):");
      o->tooltip("Enter \'..\' to complete token shown in browser");
      o->align(FL_ALIGN_TOP);
    }
    { Fl_Check_Button* o = cb_keywords = new Fl_Check_Button(475, 70, 225, 25, "Search keywords");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = cb_comment = new Fl_Check_Button(475, 95, 225, 25, "Search comment");
      o->down_box(FL_DOWN_BOX);
    }    
    { Fl_Check_Button* o = cb_title = new Fl_Check_Button(475, 120, 215, 25, "Search title");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = cb_path = new Fl_Check_Button(475, 145, 245, 25, "Search path");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = cb_groups = new Fl_Check_Button(475, 170, 255, 25, "Include groups above");
      o->down_box(FL_DOWN_BOX);
    }
    { Fl_Check_Button* o = cb_wholeword = new Fl_Check_Button(475, 195, 245, 25, "Whole word");
      o->down_box(FL_DOWN_BOX);
    }
    btn_cancel = new Fl_Button(610, 500, 155, 35, "Cancel");
    btn_mark = new Fl_Button(475, 230, 135, 35, "Mark");
    btn_search = new Fl_Return_Button(630, 230, 135, 35, "Search");
    { 
#if (BROWSER == 0)
      Fl_Browser* o = brw_kws = new Fl_Select_Browser(15, 65, 435, 470);
#else
      KWSRCHBROWSER* o = brw_kws = new TextTreeBrowser(15, 65, 435, 470, "", root);
#endif
      o->box(FL_NO_BOX);
      o->color(FL_BACKGROUND2_COLOR);
      o->selection_color(FL_SELECTION_COLOR);
      o->labeltype(FL_NORMAL_LABEL);
      o->labelfont(0);
      o->labelsize(14);
      o->labelcolor(FL_FOREGROUND_COLOR);
      o->callback((Fl_Callback*)cb_browser);
      o->align(FL_ALIGN_BOTTOM);
      o->when(FL_WHEN_RELEASE_ALWAYS);
      Fl_Group::current()->resizable(o);
    }
    o->end();
  }
  
#if (BROWSER == 0)
  {
    std::set<std::string>::iterator i = kwset.begin();
    while (i != kwset.end())
    {
      brw_kws->add((*i).c_str());
      i++;
    }    
  }
#endif

  ti_words->when(FL_WHEN_CHANGED);
  ti_words->callback((Fl_Callback*)callback_text_input_to_texttreebrowser_comma);
  ti_words->user_data((void*)brw_kws);
  brw_kws->user_data(this);
  brw_kws->textsize(14);
  
  cb_keywords->value(1);
  cb_comment ->value(1);
  cb_title   ->value(1);
  cb_groups  ->value(1);
  cb_wholeword->deactivate();
  
  printf("search dialog is %p\n", this);
  printf("browser is       %p\n", brw_kws);
  printf("input line is    %p\n", ti_words);
}

SearchDlg::~SearchDlg()
{
  delete myWindow;
}

int SearchDlg::showDlg()
{
  int nRetCode;
  
  myWindow->set_modal();
  myWindow->show();
      
  for (;;)
  {
    Fl_Widget *o = Fl::readqueue();
    
    if (!o) 
    {
      Fl::wait();
    }
    else if (o == btn_cancel || o == myWindow)
    {
      nRetCode = 0;
      break;
    }
    else if (o == btn_mark)
    {
      nRetCode = 1;
      break;
    }    
    else if (o == btn_search)
    {
      nRetCode = 2;
      break;
    }    
  }
  
  myWindow->hide();
  return(nRetCode);
}

void SearchDlg::setData(SearchData* data)
{
  data->setWords(ti_words->value());

  data->nFlags = 0;
  
  if (cb_keywords->value())
    data->nFlags |= SD_KEYWORDS;
  if (cb_comment->value())
    data->nFlags |= SD_COMMENT;
  if (cb_title->value())
    data->nFlags |= SD_TITLE;
  if (cb_path->value())
    data->nFlags |= SD_PATH;
  if (cb_groups->value())
    data->nFlags |= SD_GROUP;
}
